//*****************************************************************************
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, Inc. 
// All Rights Reserved 
//*****************************************************************************
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor: Xilinx
// \   \   \/     Version: 1.0
//  \   \         Filename: sys_top.v
//  /   /         Date Last Modified:
// /___/   /\     Date Created: 8/16/06
// \   \  /  \
//  \___\/\___\
//
//Device: Virtex-5
//Purpose:
//   Top-level  module. Simple model for what the user might use
//   Typically, the user will only instantiate MEM_INTERFACE_TOP in their 
//   code, and generate all the other infrastructure and backend logic 
//   separately. This module serves both as an example, and allows the user
//   to synthesize a self-contained design, which they can use to test their
//   hardware. 
//   In addition to the memory controller, the module instantiates:
//     1. Clock generation/distribution, reset logic
//     2. IDELAY control block
//     3. Synthesizable testbench - used to model user's backend logic
//Reference:
//   Rev 1.0 - created. adapted V5 from DDR2 SDRAM controller from MIG 1.6
//             from karthip design. richc. 8/16/06
//*****************************************************************************

`timescale 1ns/1ps

module mem_interface_top #
  (
   parameter integer BANK_WIDTH     = 2,  // # of memory bank addr bits
   parameter integer CKE_WIDTH      = 2,  // # of memory clock enable outputs
   parameter integer CLK_WIDTH      = 3,  // # of clock outputs
   parameter integer COL_WIDTH      = 10, // # of memory column bits
   parameter integer CS_BITS        = 0,  // set to log2(CS_NUM) (rounded up)
   parameter integer CS_NUM         = 1,  // # of separate memory chip selects
   parameter integer CS_WIDTH       = 2,  // # of total memory chip selects
   parameter integer DM_WIDTH       = 8,  // # of data mask bits
   parameter integer DQ_BITS        = 8,  // set to log2(DQS_WIDTH*DQ_PER_DQS)
   parameter integer DQ_PER_DQS     = 8,  // # of DQ data bits per strobe
   parameter integer ODT_WIDTH      = 2,      // # of memory on-die term enables
   parameter integer ADDITIVE_LAT   = 0,      // additive write latency
   parameter integer ECC_ENABLE     = 0,      // enable ECC (=1 enable)
   parameter integer ODT_TYPE       = 0,      // ODT (=0(none),=1(75),=2(150),=3(50))
   parameter integer DQS_BITS       = 3,  // set to log2(DQS_WIDTH)
   parameter integer DQS_WIDTH      = 8,  // # of DQS strobes
   parameter integer ROW_WIDTH      = 14, // # of memory row & # of addr bits
   parameter integer BURST_LEN      = 8,  // burst length (in double words)
   parameter integer BURST_TYPE     = 0,  // burst type (=0 seq; =1 interlved)
   parameter integer CAS_LAT        = 5,  // CAS latency
   parameter integer MULTI_BANK_EN  = 1,  // enable bank management
   parameter integer DDR2_ENABLE    = 1,      // = 1 for DDR2, = 0 for DDR1
   parameter integer REDUCE_DRV     = 0,  // reduced strength mem I/O (=1 yes)
   parameter integer REG_ENABLE     = 1,  // registered addr/ctrl (=1 yes)
   parameter integer TREFI_NS       = 7800,    // auto refresh interval (uS)
   parameter integer TRAS           = 40000,   // active->precharge delay
   parameter integer TRCD           = 15000,   // active->read/write delay
   parameter integer TRFC           = 70000,   // ref->ref, ref->active delay
   parameter integer TRP            = 15000,   // precharge->command delay
   parameter integer TRTP           = 7500,    // read->precharge delay
   parameter integer TWR            = 15000,   // used to determine wr->prech
   parameter integer TWTR           = 3,      // write->read delay
   parameter integer CLK_PERIOD     = 3000,    // Core/Mem clk period (in ps)
   parameter         DLL_FREQ_MODE  = "HIGH",  // DCM Frequency range
   parameter         IDEL_HIGH_PERF = "TRUE", // IDELAY low jitter mode
   parameter integer SIM_ONLY      = 0,        // = 1 to skip power up delay
   parameter integer DEBUG_BUILD   = 1       // include Chipscope logic

   )
  (
   // clocks/reset
   input                              sys_clk_p,
   input                              sys_clk_n,
   input                              clk200_p,
   input                              clk200_n,
   input                              sys_rst_n,
   // debug logic
   output [6:0]                       seven_seg_n,
   output                             seven_seg_dp_n,
   output [3:0]                       led,
  // output [7:0]                       debug,
   // memory bus
   output [CLK_WIDTH-1:0]             ddr2_ck,
   output [CLK_WIDTH-1:0]             ddr2_ck_n,
   output [ROW_WIDTH-1:0]             ddr2_addr,
   output [BANK_WIDTH-1:0]            ddr2_ba,
   output                             ddr2_ras_n,
   output                             ddr2_cas_n,
   output                             ddr2_we_n,
   output [CS_WIDTH-1:0]              ddr2_cs_n,
   output [CKE_WIDTH-1:0]             ddr2_cke,
   output [ODT_WIDTH-1:0]             ddr2_odt,   
   output [DM_WIDTH-1:0]              ddr2_dm,
   output                             ddr2_reset_n,
   inout [DQS_WIDTH-1:0]              ddr2_dqs,
   inout [DQS_WIDTH-1:0]              ddr2_dqs_n,
   inout [(DQS_WIDTH*DQ_PER_DQS)-1:0] ddr2_dq
   );

  localparam DQ_WIDTH = DQS_WIDTH * DQ_PER_DQS;

  // # IDELAY taps each bit period (0.5 * clock period) covers (round down for
  // now, might change to round to nearest integral tap value)
  localparam integer BT_TAPS = (CLK_PERIOD)/150;

   

  
  wire [30:0]                         app_af_addr;
  wire                                app_af_afull;
  wire [2:0]                          app_af_cmd;
  wire                                app_af_wren;
  wire                                app_wdf_afull;
  wire [(2*DQ_WIDTH)-1:0]             app_wdf_data;
  wire [(2*DM_WIDTH)-1:0]             app_wdf_mask_data;              
  wire                                app_wdf_wren;
  wire [5:0]                          bit_time_taps;
  wire [5:0]                          dq_idel_init;
  wire                                clk0;
  wire                                clk90;
  wire                                clk200;
  wire                                error;
  wire                                idelay_ctrl_rdy;
  wire                                phy_init_done;
  wire                                rd_data_valid;
  wire [(2*DQ_WIDTH)-1:0]             rd_data_fifo_out;               
  wire                                rst0;
  wire                                rst90;
  wire                                rst200;

  // debug

  wire [255:0] 		              init_calib;
  wire [255:0] 		              calib_test;
  wire [255:0]			      cmp_test;
  wire [35:0]                         control0;
  wire [255:0]                        data;
  wire [7:0]                          trig0;
   
  
  //***************************************************************************
 
   assign             ddr2_reset_n = ~rst0;
   assign             bit_time_taps = 6'd0;
   
  // IDELAY control
  idelay_ctrl  u_idelay_ctrl
    (
     .clk200(clk200),
     .rst200(rst200),
     .idelay_ctrl_rdy(idelay_ctrl_rdy)
     );

  // clocking/reset
  infrastructure #
    (
     .CLK_PERIOD      (CLK_PERIOD)
     )
    u_infrastructure
      (
       .sys_clk_p       (sys_clk_p),
       .sys_clk_n       (sys_clk_n),
       .clk200_p        (clk200_p),
       .clk200_n        (clk200_n),
       .sys_rst_n       (sys_rst_n),
       .idelay_ctrl_rdy (idelay_ctrl_rdy),
       .clk0            (clk0),
       .clk90           (clk90),
       .clk200          (clk200),
       .rst0            (rst0),
       .rst90           (rst90),
       .rst200          (rst200)
       );

  // memory initialization/control logic
  ddr2_top #
    (
 
     .BANK_WIDTH     (BANK_WIDTH),
     .CKE_WIDTH      (CKE_WIDTH),
     .CLK_WIDTH      (CLK_WIDTH),
     .COL_WIDTH      (COL_WIDTH),
     .CS_BITS        (CS_BITS),
     .CS_NUM         (CS_NUM),
     .CS_WIDTH       (CS_WIDTH),
     .DM_WIDTH       (DM_WIDTH),
     .DQ_BITS        (DQ_BITS),
     .DQ_PER_DQS     (DQ_PER_DQS),
     .DQS_BITS       (DQS_BITS),
     .DQS_WIDTH      (DQS_WIDTH),
     .ODT_WIDTH      (ODT_WIDTH),  
     .ROW_WIDTH      (ROW_WIDTH),
     .ADDITIVE_LAT   (ADDITIVE_LAT),             
     .BURST_LEN      (BURST_LEN),
     .BURST_TYPE     (BURST_TYPE),
     .CAS_LAT        (CAS_LAT),
     .ECC_ENABLE     (0),              // not currently supported
     .MULTI_BANK_EN  (MULTI_BANK_EN),
     .ODT_TYPE       (ODT_TYPE),             
     .REDUCE_DRV     (REDUCE_DRV),
     .REG_ENABLE     (REG_ENABLE),
     .TREFI_NS       (TREFI_NS),
     .TRAS           (TRAS),
     .TRCD           (TRCD),
     .TRFC           (TRFC),
     .TRP            (TRP),
     .TRTP           (TRTP),
     .TWR            (TWR),
     .TWTR           (TWTR),              
     .CLK_PERIOD     (CLK_PERIOD),
     .DDR2_ENABLE    (DDR2_ENABLE),              
     .DQS_GATE_EN    (0),              // disable DQS gate circuit for DDR2
     .IDEL_HIGH_PERF (IDEL_HIGH_PERF),
     .SIM_ONLY       (SIM_ONLY)

     )
    u_ddr2_top
      (
       .clk0              (clk0),
       .clk90             (clk90),
       .rst0              (rst0),
       .rst90             (rst90),
       .reg_addr          (reg_addr),
       .dq_idel_init      (dq_idel_init),
       .bit_time_taps     (bit_time_taps),
       .app_af_cmd        (app_af_cmd),
       .app_af_addr       (app_af_addr),
       .app_af_wren       (app_af_wren),
       .app_wdf_wren      (app_wdf_wren),
       .app_wdf_data      (app_wdf_data),
       .app_wdf_mask_data (app_wdf_mask_data),
       .app_af_afull      (app_af_afull),
       .app_wdf_afull     (app_wdf_afull),
       .rd_data_valid     (rd_data_valid),
       .rd_data_fifo_out  (rd_data_fifo_out),
       .phy_init_done     (phy_init_done),
       .ddr_ck            (ddr2_ck),
       .ddr_ck_n          (ddr2_ck_n),
       .ddr_addr          (ddr2_addr),
       .ddr_ba            (ddr2_ba),
       .ddr_ras_n         (ddr2_ras_n),
       .ddr_cas_n         (ddr2_cas_n),
       .ddr_we_n          (ddr2_we_n),
       .ddr_cs_n          (ddr2_cs_n),
       .ddr_cke           (ddr2_cke),
       .ddr_odt           (ddr2_odt), 
       .ddr_dm            (ddr2_dm),
       .ddr_dqs           (ddr2_dqs),
       .ddr_dqs_n         (ddr2_dqs_n),
       .ddr_dq            (ddr2_dq),
       .reg_rdata         (reg_rdata),
       .calib_test        (calib_test),
       .init_calib        (init_calib)
       );

  // synthesizable testbench
  tb_top #
    (
     .BANK_WIDTH (BANK_WIDTH),
     .COL_WIDTH  (COL_WIDTH),
     .DM_WIDTH   (DM_WIDTH),
     .DQ_WIDTH   (DQ_WIDTH),
     .ROW_WIDTH  (ROW_WIDTH),
     .BURST_LEN  (BURST_LEN)
     )
    u_tb_top
      (
       .clk               (clk0),
       .rst               (rst0),
       .app_af_afull      (app_af_afull),
       .app_wdf_afull     (app_wdf_afull),
       .rd_data_valid     (rd_data_valid),
       .rd_data_fifo_out  (rd_data_fifo_out),
       .phy_init_done     (phy_init_done),
       .app_af_wren       (app_af_wren),
       .app_af_cmd        (app_af_cmd),
       .app_af_addr       (app_af_addr),
       .app_wdf_wren      (app_wdf_wren),
       .app_wdf_data      (app_wdf_data),
       .app_wdf_mask_data (app_wdf_mask_data),
       .cmp_test          (cmp_test),
       .error             (error)
       );
     led_display_driver u_led_display_driver    
    (
     .rst            (rst0),
     .clk            (clk0),
     .init_done_i    (phy_init_done), 
     .error_i        (error),
     .valid_i        (rd_data_valid),
     .seven_seg_n    (seven_seg_n),
     .seven_seg_dp_n (seven_seg_dp_n),
     .led            (led)
     );

 generate
   if(DEBUG_BUILD)begin: gen_debug
  //***************************************************************************
  // Debug - remove for final code
  //***************************************************************************

 
  
  //*****************************************************************
  // Internal debug
  //*****************************************************************


 //-----------------------------------------------------------------
  //
  //  ILA Core wire declarations
  //
  //-----------------------------------------------------------------



assign trig0 = {init_calib[7:0]};
assign data = {calib_test};
  //-----------------------------------------------------------------
  //
  //  ILA core instance
  //
  //-----------------------------------------------------------------
  ila i_ila
    (
      .control(control0),
      .clk(clk0),
      .data(data),
      .trig0(trig0)
    );


 //-----------------------------------------------------------------
  //
  //  ICON core instance
  //
  //-----------------------------------------------------------------
  icon i_icon
    (
      .control0(control0)
    );



    



 
 
   end // if (DEBUG_BUILD)
 
 endgenerate


endmodule

//-------------------------------------------------------------------
//
//  ILA core module declaration
//
//-------------------------------------------------------------------
module ila
  (
    control,
    clk,
    data,
    trig0
  );
  input [35:0] control;
  input clk;
  input [255:0] data;
  input [7:0] trig0;
endmodule


//-------------------------------------------------------------------
//
//  ICON core module declaration
//
//-------------------------------------------------------------------
module icon
  (
      control0
  );
  output [35:0] control0;
endmodule
