//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2007 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.1
//  \   \              Filename: raid6_block.v 
//  /   /              Date Last Modified: 2/23/2007
// /___/   /\          Date Created: 10/6/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: raid6_block 
//Purpose:  This module is the top module of the RAID6 ip it contains the logic
//          to perform RAID5 and RAID6 calculations on 4byte data width.  It is
//          coupled with a modified GSRD design.
//          This is part of XAPP 731
//
//           This is also part of XAPP 865 and modified for the different application
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
`include "../rtl/raid6_parameters.v"

module raid6_block(
input  wire        clk,
input  wire        clk_mem, 
input  wire        rst,
input  wire        start,
output reg   [1:0] DCR_sel,
input  wire [31:0] DCR_reg0,
///////////////////////////////////////////////////////////////////////////////
//// this has several DCR registers mux'd
//// including Mode(shown below), Q, P
//// 000 1 D lost
//// 001   P lost
//// 010   Q lost
//// 011  UPDATE P and Q for new data 
//// 100 2 D lost
//// 101  DP lost
//// 110  DQ lost
//// 111  QP lost
///////////////////////////////////////////////////////////////////////////////
input  wire [31:0] DCR_reg1,
///////////////////////////////////////////////////////////////////////////////
//// this has several DCR registers mux'd
//// including one hot encoding of which are lost
//// plus D1, D2, and D3 address locations
///////////////////////////////////////////////////////////////////////////////
input  wire [31:0] RAID_SIZE,
output wire        SINGLE_ERR,
output wire        DOUBLE_ERR,
///////////////////////////////////////////////////////////////////////////////
//// this determines the block size and how many sequences need to be run through
//// so that the whole block is calculated
//// one hot encoding indicated what is lost
//// D3,D2,D1,D0,P,Q
///////////////////////////////////////////////////////////////////////////////
output reg         done,
///////////////////////////////////////////////////////////////////////////////
////indicated when the generation/regeneration is complete
///////////////////////////////////////////////////////////////////////////////
output wire        gfc_done, 
///////////////////////////////////////////////////////////////////////////////
//// gfc_done indicates that the BRAM are initialized
////no longer active but can be used if generating new polynomial
///////////////////////////////////////////////////////////////////////////////
output wire [`raid_width-1:0] mem_wrdata,
output wire [15:0] mem_wr_parity,
input  wire [`raid_width-1:0] mem_rddata,
input  wire [15:0] mem_rd_parity,
output wire [35:0] mem_addr,
output wire        mem_data_wen,
output wire        mem_addr_wen,   
input  wire        mem_data_valid  

);

 //synthesis attribute keep clk "true";


//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//VARIABLE PARAMETERS
// 
//this parameter determines the block size that is used
// 
//
//////////////////////////////////////////////////////////////////////////
parameter DATABLOCKSIZE = 10'h040;   

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//registers that indicate information for what disk is lost 
//and what combination
reg  [10:0] LOST;
reg  [31:0] D0_ADDR;
reg  [31:0] D1_ADDR;
reg  [31:0] D2_ADDR;
reg  [31:0] D3_ADDR;
reg  [31:0] P_ADDR;
reg  [31:0] Q_ADDR;
reg   [2:0] RAID_MODE; //defined in port list
 
wire [`raid_width-1:0] xor_data_0;
wire [`raid_width-1:0] xor_data_1;
wire [`raid_width-1:0] xor_data_2;
wire [`raid_width-1:0] xor_data_3;
wire [`raid_width-1:0] xor_data_4;
wire [`raid_width-1:0] mux_data_0;
wire [`raid_width-1:0] mux_data_1;
wire [`raid_width-1:0] mux_data_2;
wire [`raid_width-1:0] mux_data_3;
wire [`raid_width-1:0] mux_data_4;
wire [`raid_width-1:0] mult_data_0;
wire [`raid_width-1:0] mult_data_1;
wire [`raid_width-1:0] mult_data_2;
wire [`raid_width-1:0] mult_data_3;
wire [`raid_width-1:0] bram_data_0;
wire  [7:0] dspin_0;
wire  [7:0] dspin_1;
wire  [7:0] dspin_2;
wire  [7:0] dspin_3;
wire [`raid_width-1:0] xor_bramdatain;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//the state machine signals start here
//
//////////////////////////////////////////////////////////////////////////
//control the mux that feed the final GF function block
reg         xor_bypass;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//controls the input of the XOR_BRAM block
//( loopback saves time in certain modes)
reg         feedback_loop;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//controls the input of the xor_data_3 when DD case is being solved
reg         dd_lost_mux;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//keeps track of number of xors have been done
reg   [1:0] block_cnt;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//determines what data to write to the memory
reg   [1:0] memorydata_sel;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//controls the input into the XOR_BRAM block
reg         xor_muxsel;
//controls the BRAM addr of the XOR_BRAM block
reg  [11:0] xor_addrin;
reg  [11:0] xor_addrout;
reg         xor_wren;
reg         xor_ren;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//A controls the input into the BRAM of the BRAM_MUX blocks
//B controls the output of the BRAM_MUX blocks
reg         bram0_muxsel_a;        
reg         bram0_muxsel_b; 
reg         bram1_muxsel_a;        
reg         bram1_muxsel_b;
reg         bram2_muxsel_a;        
reg         bram2_muxsel_b;
reg         bram3_muxsel_a;
//controls the BRAM addr of the BRAM_MUX blocks
reg  [11:0] bram0_addrin;
reg  [11:0] bram0_addrout;
reg  [11:0] bram1_addrin;
reg  [11:0] bram1_addrout;
reg  [11:0] bram2_addrin;
reg  [11:0] bram2_addrout;
reg  [11:0] bram3_addrin;
reg  [11:0] bram3_addrout;
//controls for the BRAM of the BRAM_MUX blocks
reg         bram0_wren;
reg         bram0_ren;
reg         bram1_wren;
reg         bram1_ren;
reg         bram2_wren;
reg         bram2_ren;
reg         bram3_wren;
reg         bram3_ren;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//controls the GF addition and multiplication blocks of the RAID_MULT4
reg   [2:0] select_0;
reg   [2:0] select_1;
reg   [2:0] select_2;
reg   [3:0] select_3;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//other handshaking signals
reg         cycle_done;
reg         DCR_done;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//raid to mpmc controller signals
reg  [`raid_width-1:0] mem_datain;
reg  [`raid_width-1:0] mem_datain_reg;
wire        raid_addrack;
reg         raid_req;
reg  [31:0] raid_addr;
reg         raid_RNW;
wire [`raid_width-1:0] raid_wrdata;
reg  [10:0] sector_len;  
wire        raid_wrrdy;
wire        raid_dv;
wire [`raid_width-1:0] raid_rddata;
wire        raid_complete;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//this is the statemachine that controls the RAID circuitry
reg   [7:0] state;

//////////////////////////////////////////////////////////////////////////
//these parameters are the states for the main state machine
//IDLE determines if SINGLE or DOUBLE regeneration is needed
//then goes to that state where it is decided which regeneration
//is needed.  The states have prefixs that indicate which regeneration
//is taking place. 
//REQ is reading from the memory  
//LOAD states are writing data back to the memory
//XOR uses the XOR block
//MD used the MUX_BRAM and MULT_RAID_4 blocks
//////////////////////////////////////////////////////////////////////////

parameter   IDLE           = 8'b00000000;
parameter   SINGLE         = 8'b00000001;
parameter   DOUBLE         = 8'b01000000;
parameter   PREQDATA       = 8'b00000010;
parameter   PREQDATA1      = 8'b00000100;
parameter   PXORBWRITE     = 8'b00000011;
parameter   PXOR           = 8'b00000101;
parameter   PXOR1          = 8'b00000110;
parameter   PXOR2          = 8'b00000111;
parameter   PXOR3          = 8'b00001000;
parameter   PLOAD          = 8'b00001001;
parameter   PLOAD1         = 8'b00001010;
parameter   PLOAD2         = 8'b00001011;
parameter   PLOAD3         = 8'b00001100;
parameter   PLOAD4         = 8'b00001101;
parameter   PLOAD5         = 8'b00001110;
parameter   QREQDATA       = 8'b00001111;
parameter   QMBWRITE       = 8'b00010000;
parameter   QMBWRITE1      = 8'b00010001;
parameter   QMBWRITE2      = 8'b00010010;
parameter   QREQDATA1      = 8'b00010011;
parameter   QMBWRITE3      = 8'b00010100;
parameter   QMBWRITE4      = 8'b00010101;
parameter   QMBWRITE5      = 8'b00010110;
parameter   QREQDATA2      = 8'b00010111;
parameter   QMBWRITE6      = 8'b00011000;
parameter   QMBWRITE7      = 8'b00011001;
parameter   QMBWRITE8      = 8'b00011010;
parameter   QLOAD          = 8'b00011011;
parameter   QLOAD1         = 8'b00011100;
parameter   QLOAD2         = 8'b00011101;
parameter   QLOAD3         = 8'b00011110;
parameter   QLOAD4         = 8'b00011111;
parameter   DREQDATA       = 8'b00100000;
parameter   DREQDATA1      = 8'b00100010;
parameter   DXORBWRITE     = 8'b00100001;
parameter   DXOR           = 8'b00100011;
parameter   DXOR1          = 8'b00100100;
parameter   DXOR2          = 8'b00100101;
parameter   DXOR3          = 8'b00100110;
parameter   DLOAD          = 8'b00100111;
parameter   DLOAD1         = 8'b00101000;
parameter   DLOAD2         = 8'b00101001;
parameter   DLOAD3         = 8'b00101010;
parameter   DLOAD4         = 8'b00101011;
parameter   DLOAD5         = 8'b00101100;
parameter   PQREQDATA      = 8'b01000001;
parameter   PQXORMBWRITE   = 8'b01000010;
parameter   PQXORMBWRITE1  = 8'b01000011;
parameter   PQXORMBWRITE2  = 8'b01000100;
parameter   PQREQDATA1     = 8'b01000101;
parameter   PQXORMBWRITE3  = 8'b01000110;
parameter   PQXORMBWRITE4  = 8'b01000111;
parameter   PQXORMBWRITE5  = 8'b01001000;
parameter   PQXORMBWRITE6  = 8'b01001001;
parameter   PQREQDATA2     = 8'b01001010;
parameter   PQXORMBWRITE7  = 8'b01001011;
parameter   PQXORMBWRITE8  = 8'b01001100;
parameter   PQXORMBWRITE9  = 8'b01001101;
parameter   PQXORMBWRITE10 = 8'b01001110;
parameter   PQLOAD         = 8'b01001111;
parameter   PQLOAD1        = 8'b01010000;
parameter   PQLOAD2        = 8'b01010001;
parameter   PQLOAD3        = 8'b01010010;
parameter   PQLOAD4        = 8'b01010011;
parameter   PQLOAD5        = 8'b01010100;
parameter   PQLOAD6        = 8'b01010101;
parameter   PQLOAD7        = 8'b01010110;
parameter   PQLOAD8        = 8'b01010111;
parameter   PQLOAD9        = 8'b01011000;
parameter   PQLOAD10       = 8'b01011001;
parameter   PQLOAD11       = 8'b01011010;
parameter   DQREQDATA      = 8'b01011011;
parameter   DQXORBWRITE    = 8'b01011100;
parameter   DQREQDATA1     = 8'b01011101;
parameter   DQXORMBWRITE   = 8'b01011110;
parameter   DQXORMBWRITE1  = 8'b01011111;
parameter   DQXORMBWRITE2  = 8'b01100000;
parameter   DQXORMBWRITE3  = 8'b01100001;
parameter   DQREQDATA2     = 8'b01100010;
parameter   DQXORMBWRITE4  = 8'b01100011;
parameter   DQXORMBWRITE5  = 8'b01100100;
parameter   DQXORMBWRITE6  = 8'b01100101;
parameter   DQXORMBWRITE7  = 8'b01100110;
parameter   DQLOAD         = 8'b01100111;
parameter   DQLOAD1        = 8'b01101000;
parameter   DQLOAD2        = 8'b01101001;
parameter   DQLOAD3        = 8'b01101010;

parameter   DQLOAD3A       = 8'b11111110;

parameter   DQLOAD4        = 8'b01101011;
parameter   DQMBWRITE      = 8'b01101100;
parameter   DQMBWRITE1     = 8'b01101101;
parameter   DQLOAD5        = 8'b01101110;
parameter   DQLOAD6        = 8'b01101111;
parameter   DQLOAD7        = 8'b01110000;
parameter   DQLOAD8        = 8'b01110001;
parameter   DQLOAD9        = 8'b01110010;
parameter   PDREQDATA      = 8'b01110011;
parameter   PDXORMBWRITE   = 8'b01110100;
parameter   PDXORMBWRITE1  = 8'b01110101;
parameter   PDXORMBWRITE2  = 8'b01110110;
parameter   PDREQDATA1     = 8'b01110111;
parameter   PDXORMBWRITE3  = 8'b01111000;
parameter   PDXORMBWRITE4  = 8'b01111001;
parameter   PDXORMBWRITE5  = 8'b01111010;
parameter   PDXORMBWRITE6  = 8'b01111011;
parameter   PDREQDATA2     = 8'b01111100;
parameter   PDXORMBWRITE7  = 8'b01111101;
parameter   PDXORMBWRITE8  = 8'b01111110;
parameter   PDXORMBWRITE9  = 8'b01111111;
parameter   PDLOAD         = 8'b10000000;
parameter   PDLOAD1        = 8'b10000001;
parameter   PDLOAD2        = 8'b10000010;
parameter   PDLOAD3        = 8'b10000011;
parameter   PDLOAD4        = 8'b10000100;
parameter   PDLOAD5        = 8'b10000101;
parameter   PDLOAD6        = 8'b10000110;
parameter   PDLOAD7        = 8'b10000111;
parameter   PDLOAD8        = 8'b10001000;
parameter   PDLOAD9        = 8'b10001001;
parameter   PDLOAD10       = 8'b10001010;
parameter   PDLOAD11       = 8'b10001011;
parameter   DDREQDATA      = 8'b10001100;
parameter   DDXORMBWRITE   = 8'b10001101;
parameter   DDXORMBWRITE1  = 8'b10001110;
parameter   DDXORMBWRITE2  = 8'b10001111;
parameter   DDREQDATA1     = 8'b10010000;
parameter   DDXOR          = 8'b10010001;
parameter   DDXOR1         = 8'b10010010;
parameter   DDXOR2         = 8'b10010011;
parameter   DDXOR3         = 8'b10010100;
parameter   DDREQDATA2     = 8'b10010101;
parameter   DDMBWRITE      = 8'b10010110;
parameter   DDMBWRITE1     = 8'b10010111;
parameter   DDMBWRITE2     = 8'b10011000;
parameter   DDCALCDATA     = 8'b10011001;
parameter   DDCALCDATA1    = 8'b10011010;
parameter   DDCALCDATA2    = 8'b10011011;
parameter   DDCALCDATA3    = 8'b10011100;
parameter   DDCALCDATA4    = 8'b10011101;
parameter   DDCALCDATA5    = 8'b10011110;
parameter   DDCALCDATA6    = 8'b10011111;
parameter   DDLOAD         = 8'b10100000;
parameter   DDLOAD1        = 8'b10100001;
parameter   DDLOAD2        = 8'b10100010;
parameter   DDLOAD3        = 8'b10100011;
parameter   DDLOAD4        = 8'b10100100;
parameter   DDLOAD5        = 8'b10100101;
parameter   DDLOAD6        = 8'b10100110;
parameter   DDLOAD7        = 8'b10100111;
parameter   DDLOAD8        = 8'b10101000;
parameter   DDLOAD9        = 8'b10101001;
parameter   DDLOAD10       = 8'b10101010;
parameter   DDLOAD11       = 8'b10101011;

parameter   UPDATE         = 8'b10101100;
parameter   UREQDATA       = 8'b10101101;
parameter   UXORMBWRITE    = 8'b10101110;
parameter   UXORMBWRITE1   = 8'b10101111;
parameter   UXORMBWRITE2   = 8'b10110000;
parameter   UREQDATA1      = 8'b10110001;
parameter   UXORMBWRITE3   = 8'b10110010;
parameter   UXORMBWRITE4   = 8'b10110011;
parameter   UXORMBWRITE5   = 8'b10110100;
parameter   UXORMBWRITE6   = 8'b10110101;
parameter   UREQDATA2      = 8'b10110110;
parameter   UXORMBWRITE7   = 8'b10110111;
parameter   UXORMBWRITE8   = 8'b10111000;
parameter   UXORMBWRITE9   = 8'b10111001;
parameter   ULOAD          = 8'b10111011;
parameter   ULOAD1         = 8'b10111100;
parameter   ULOAD2         = 8'b10111101;
parameter   ULOAD3         = 8'b10111110;
parameter   ULOAD4         = 8'b10111111;
parameter   ULOAD5         = 8'b11000000;
parameter   UREQDATA3      = 8'b11000001;
parameter   UXORMBWRITE10  = 8'b11000010;
parameter   UXORMBWRITE11  = 8'b11000011;
parameter   UXORMBWRITE12  = 8'b11000100;
parameter   ULOAD6         = 8'b11000101;
parameter   ULOAD7         = 8'b11000110;
parameter   ULOAD8         = 8'b11000111;
parameter   ULOAD9         = 8'b11001000;
parameter   ULOAD10        = 8'b11001001;
parameter   ULOAD11        = 8'b11001010;

parameter   DONE           = 8'b11111111;

always @ (posedge clk or posedge rst)
begin
  if(rst)
  begin
    xor_bypass     <= 1'b0;
    xor_muxsel     <= 1'b0;
    xor_addrin     <= 11'h000;
    xor_addrout    <= 11'h000;
    xor_wren       <= 1'b0;
    xor_ren        <= 1'b0;
    bram0_muxsel_a <= 1'b0;        
    bram0_muxsel_b <= 1'b0; 
    bram1_muxsel_a <= 1'b0;        
    bram1_muxsel_b <= 1'b0;
    bram2_muxsel_a <= 1'b0;        
    bram2_muxsel_b <= 1'b0;
    bram3_muxsel_a <= 1'b1;
    bram0_addrin   <= 11'h000;
    bram0_addrout  <= 11'h000;
    bram1_addrin   <= 11'h000;
    bram1_addrout  <= 11'h000;
    bram2_addrin   <= 11'h000;
    bram2_addrout  <= 11'h000;
    bram3_addrin   <= 11'h000;
    bram3_addrout  <= 11'h000;
    bram0_wren     <= 1'b0;
    bram0_ren      <= 1'b0;
    bram1_wren     <= 1'b0;
    bram1_ren      <= 1'b0;
    bram2_wren     <= 1'b0;
    bram2_ren      <= 1'b0;
    bram3_wren     <= 1'b0;
    bram3_ren      <= 1'b0;
    select_0       <= 3'b000;
    select_1       <= 3'b000;
    select_2       <= 3'b000;
    select_3       <= 4'b0000;
    raid_req       <= 1'b0;
    raid_addr      <= 32'h00000000;
    raid_RNW       <= 1'b1;
    block_cnt      <= 2'b00;
    memorydata_sel <= 2'b00;
    mem_datain_reg <= 64'h0000000000000000;
    mem_datain     <= 64'h0000000000000000;
    feedback_loop  <= 1'b0;
    dd_lost_mux    <= 1'b0;
    cycle_done     <= 1'b0;
    state          <= IDLE;
  end
  else
  begin
    case (state)
      IDLE:    begin
                 if(DCR_done)
                 begin
                   if(RAID_MODE[2] == 1'b0)  //only one disk lost
                   begin
                     if(RAID_MODE[1:0] == 2'b11) //update P & Q
                     begin
                       state <= UPDATE;
                     end
                     else
                     begin
                       state <= SINGLE;
                     end
                   end
                   else  //two disks lost
                   begin
                     state <= DOUBLE;
                   end
                 end
                 else
                 begin
                   state   <= IDLE;
                 end
               end
      SINGLE:  begin
                 case(LOST)
                     //NEED to calc Q
                     11'b00000000001:  begin
                                         state     <= QREQDATA;
                                         raid_addr <= D0_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                         bram0_muxsel_b <= 1'b0;
                                         bram0_wren     <= 1'b1;
                                       end 
                     //NEED to calc P 
                     11'b00000000010:  begin
                                         state     <= PREQDATA;
                                         raid_addr <= D0_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                       end 
                     //NEED to calc D0 
                     11'b00000000100:  begin
                                         state     <= DREQDATA;
                                         raid_addr <= D1_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                       end
                     //NEED to calc D1  
                     11'b00000001000:  begin
                                         state     <= DREQDATA;
                                         raid_addr <= D2_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                       end 
                     //NEED to calc D2 
                     11'b00000010000:  begin
                                         state     <= DREQDATA;
                                         raid_addr <= P_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                       end 
                     //NEED to calc D3 //not yet supported needed for raid5
                     11'b00000100000:  begin
                                         state     <= DREQDATA;
                                         raid_addr <= P_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;

                                       end
                     //NEED to calc D4 //not supported in this design 
                     //11'b00001000000:  begin
                     //                    state     <= DREQDATA;
                     //                    raid_addr <= P_ADDR;
                     //                    raid_req  <= 1'b1;
                     //                    raid_RNW  <= 1'b1;
                     //                    xor_muxsel <= 1'b0;
                     //                    xor_wren   <= 1'b1;
                     //                  end 
                     //not valid 
                            default:   begin
                                         state <= IDLE;
                                       end
                 endcase
               end
     PREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= PXORBWRITE;
                 end
                 else
                 begin
                   state <= PREQDATA;
                 end                 
               end
   PXORBWRITE: begin
                 if(raid_dv)
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   mem_datain <= raid_rddata;
                   state      <= PXORBWRITE;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   state      <= PREQDATA1;
                   xor_wren   <= 1'b0;
                 end
               end
    PREQDATA1: begin  
                 state <= PXOR;
                 if(block_cnt == 2'b00)
                 begin
                   raid_addr <= D1_ADDR;
                 end
                 else if (block_cnt == 2'b01)
                 begin
                   raid_addr <= D2_ADDR;
                 end
                 else if (block_cnt == 2'b10)
                 begin
                   raid_addr <= D3_ADDR;
                 end
                 else 
                 begin
                   raid_addr <= raid_addr;
                 end
                 raid_req    <= 1'b1;
                 raid_RNW    <= 1'b1;
                 xor_muxsel  <= 1'b1;
                 xor_ren     <= 1'b1;
                 xor_addrout <= 11'h000;
                 xor_addrin  <= 11'h000;
                 xor_wren    <= 1'b0;
               end
      PXOR:    begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   xor_addrout <= xor_addrout + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= PXOR1;
                 end
                 else
                 begin
                   state <= PXOR;
                 end                 
               end
      PXOR1:   begin
                 xor_addrout <= xor_addrout + 1'b1;
                 mem_datain  <= raid_rddata;
                 xor_wren    <= 1'b1;
                 state       <= PXOR2;
               end
      PXOR2:   begin
                 if(raid_dv)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_addrin  <= xor_addrin + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= PXOR2;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   xor_ren    <= 1'b0;
                   state      <= PXOR3;
                 end
               end
       PXOR3:  begin
                 xor_wren   <= 1'b0;
                 xor_addrin <= 11'h000;
                 if(block_cnt == 2'b01) 
/////////////////////////////////////////////////////////////////////////////////////////
                  //this should become a parameter so we can do RAID5 or RAID6 DEMOS.
                  //01 is set up for RAID6 with 3 data disks, 10 would be for RAID5 with 
                  //4 data drives
                  // if more drives then this would obviously have to get larger
////////////////////////////////////////////////////////////////////////////////////////
                 begin
                   state          <= PLOAD;
                   block_cnt      <= 2'b00;
                   memorydata_sel <= 2'b01;
                 end
                 else
                 begin
                   state     <= PREQDATA1;
                   block_cnt <= block_cnt + 1'b1;
                 end
               end
      PLOAD:   begin
                 state       <= PLOAD1;
                 raid_addr   <= P_ADDR;
                 raid_req    <= 1'b1;
                 raid_RNW    <= 1'b0;
                 xor_addrout <= 11'h000;
                 xor_ren     <= 1'b1;
               end
      PLOAD1:  begin
                 if(raid_addrack)
                 begin
                   state <= PLOAD2;
                 end
                 else
                 begin
                   state <= PLOAD1;
                 end 
               end
      PLOAD2:  begin
                 state <= PLOAD3;
                 
               end
      PLOAD3:  begin
                 state <= PLOAD4;
                 xor_addrout <= xor_addrout + 1'b1;
               end
      PLOAD4:  begin
                 state <= PLOAD5;
                 xor_addrout <= xor_addrout + 1'b1;
               end
      PLOAD5:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PLOAD5;
                 end
                 else
                 begin
                   state       <= DONE;
                   cycle_done  <= 1'b1;
                   xor_ren     <= 1'b0;
                   xor_addrout <= 11'h000;
                 end
               end
     QREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= QMBWRITE;
                 end
                 else
                 begin
                   state <= QREQDATA;
                 end                 
               end
     QMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= QMBWRITE;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   state        <= QMBWRITE1;
                   bram0_wren   <= 1'b0;
                 end
               end
    QMBWRITE1: begin
                 bram0_addrin   <= 11'h000;
                 select_1       <= 3'b000;
                 bram0_ren      <= 1'b1;
                 bram0_muxsel_a <= 1'b0;
                 state <= QMBWRITE2;
               end
    QMBWRITE2: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren    <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state        <= QMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrin  <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren    <= 1'b1;
                     state         <= QMBWRITE2;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_addr     <= D1_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren     <= 1'b1;
                     state          <= QREQDATA1;
                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren    <= 1'b1;
                   state         <= QMBWRITE2;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= QMBWRITE2;
                 end                
               end
    QREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= QMBWRITE3;
                 end
                 else
                 begin
                   state <= QREQDATA1;
                 end                 
               end
    QMBWRITE3: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= QMBWRITE3;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   state        <= QMBWRITE4;
                   bram2_wren   <= 1'b0;
                 end
               end
   QMBWRITE4: begin
                 bram2_addrin <= 11'h000;
                 select_2     <= 3'b001;
                 bram2_ren    <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state          <= QMBWRITE5;
               end
   QMBWRITE5: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= QMBWRITE5;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= QMBWRITE5;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram1_wren    <= 1'b0;
                     raid_addr     <= D2_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren     <= 1'b1;
                     state          <= QREQDATA2;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram1_wren     <= 1'b1;
                   bram0_addrout  <= bram0_addrout + 1'b1;
                   state          <= QMBWRITE5;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= QMBWRITE5;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren     <= 1'b1;
                   state         <= QMBWRITE5;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= QMBWRITE5;
                 end                
               end
   QREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= QMBWRITE6;
                 end
                 else
                 begin
                   state <= QREQDATA2;
                 end                 
               end
    QMBWRITE6: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= QMBWRITE6;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   state        <= QMBWRITE7;
                   bram2_wren   <= 1'b0;
                 end
               end
    QMBWRITE7: begin
                 bram2_addrin <= 11'h000;
                 select_2     <= 3'b010;
                 bram2_ren    <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state          <= QMBWRITE8;
               end
   QMBWRITE8: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= QMBWRITE8;
                   end
                  else
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= QMBWRITE8;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     //use this if only three data drives
                     bram2_wren <= 1'b0;
                     raid_addr  <= Q_ADDR;
                     raid_req   <= 1'b1;
                     raid_RNW    <= 1'b0;
                     bram1_addrout <= 11'h000;
                     state          <= QLOAD;
                     memorydata_sel <= 2'b10;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram0_muxsel_b <= 1'b0;  
                     bram2_wren     <= 1'b0;
                     bram1_wren     <= 1'b0;
                     bram1_ren      <= 1'b1;

                     //if 4 drives this would continue
                     //bram2_wren <= 1'b0;
                     //raid_addr  <= D3_ADDR;
                     //raid_req   <= 1'b1;
                     //raid_RNW   <= 1'b1;
                     //state          <= QREQDATA3;
                     //bram2_muxsel_a <= 1'b1;
                     //bram2_muxsel_b <= 1'b0;
                     //bram0_muxsel_b <= 1'b1;  //this allows bram1 data to get to the apprpriate XOR
                     //bram2_wren     <= 1'b1;
                     //bram1_wren     <= 1'b0;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_wren    <= 1'b1;
                   bram1_addrout <= bram1_addrout + 1'b1;
                   state         <= QMBWRITE8;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_addrout <= bram1_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= QMBWRITE8;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_ren     <= 1'b1;
                   state         <= QMBWRITE8;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= QMBWRITE8;
                 end                
               end
// not needed unless more drives 
//     QREQDATA3: begin
//                 if(raid_dv)
//                 begin
//                   raid_req   <= 1'b0;
//                   raid_RNW   <= 1'b0;
//                   mem_datain <= raid_rddata;
//                   //state    <= QMBWRITE6;
//                 end
//                 else
//                 begin
//                  // state <= QREQDATA1;
//                 end                 
//               end
//
      QLOAD:  begin
                 if(raid_addrack)
                 begin
                   state <= QLOAD1;
                 end
                 else
                 begin
                   state <= QLOAD;
                 end 
               end
      QLOAD1:  begin
                 state <= QLOAD3;
               end
      QLOAD2:  begin
                 state         <= QLOAD3;
               end
      QLOAD3:  begin
                 state         <= QLOAD4;
                 bram1_addrout <= bram1_addrout + 1'b1;
               end
      QLOAD4:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   state         <= QLOAD4;
                 end
                 else
                 begin
                   state         <= DONE;
                   cycle_done    <= 1'b1;
                   bram1_ren     <= 1'b0;
                   bram1_addrout <= 11'h000;
                   bram2_addrout <= 11'h000;
                   bram0_addrout <= 11'h000;
                   bram1_addrin  <= 11'h000;
                   bram0_addrin  <= 11'h000;
                 end
               end
     DREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= DXORBWRITE;
                 end
                 else
                 begin
                   state <= DREQDATA;
                 end                 
               end
   DXORBWRITE: begin
                 if(raid_dv)
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   mem_datain <= raid_rddata;
                   state      <= DXORBWRITE;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   state      <= DREQDATA1;
                   xor_wren   <= 1'b0;
                 end
               end
    DREQDATA1: begin
                 state <= DXOR;
                 if(LOST == 11'b00000000100) //D0 lost
                 begin
                   if(block_cnt == 2'b00)
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   else if (block_cnt == 2'b01)
                   begin
                     raid_addr <= P_ADDR;
                   end
                   else if (block_cnt == 2'b10)
                   begin
                     raid_addr <= D3_ADDR;
                   end
                   else 
                   begin
                     raid_addr <= raid_addr;
                   end
                 end
                 else if(LOST == 11'b00000001000) //D1 lost
                 begin
                   if(block_cnt == 2'b00)
                   begin
                     raid_addr <= P_ADDR;
                   end
                   else if (block_cnt == 2'b01)
                   begin
                     raid_addr <= D0_ADDR;
                   end
                   else if (block_cnt == 2'b10)
                   begin
                     raid_addr <= D3_ADDR;
                   end
                   else 
                   begin
                     raid_addr <= raid_addr;
                   end
                 end
                 else if(LOST == 11'b00000010000) //D2 lost
                 begin
                   if(block_cnt == 2'b00)
                   begin
                     raid_addr <= D0_ADDR;
                   end
                   else if (block_cnt == 2'b01)
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (block_cnt == 2'b10)
                   begin
                     raid_addr <= D3_ADDR;
                   end
                   else 
                   begin
                     raid_addr <= raid_addr;
                   end
                 end
                 else if(LOST == 11'b00000100000) //D3 lost
                 begin
                   if(block_cnt == 2'b00)
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (block_cnt == 2'b01)
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   else if (block_cnt == 2'b10)
                   begin
                     raid_addr <= D3_ADDR;
                   end
                   else 
                   begin
                     raid_addr <= raid_addr;
                   end
                 end
                 else //never should run into this case
                 begin
                   if(block_cnt == 2'b00)
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (block_cnt == 2'b01)
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   else if (block_cnt == 2'b10)
                   begin
                     raid_addr <= D3_ADDR;
                   end
                   else 
                   begin
                     raid_addr <= raid_addr;
                   end
                 end
                 raid_req <= 1'b1;
                 raid_RNW <= 1'b1;
                 xor_muxsel <= 1'b1;
                 xor_ren <= 1'b1;
                 xor_addrout <= 11'h000;
                 xor_addrin <= 11'h000;
                 xor_wren <= 1'b0;
               end
      DXOR:    begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   xor_addrout <= xor_addrout + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= DXOR1;
                 end
                 else
                 begin
                   state <= DXOR;
                 end                 
               end
      DXOR1:   begin
                 xor_addrout <= xor_addrout + 1'b1;
                 mem_datain  <= raid_rddata;
                 xor_wren    <= 1'b1;
                 state       <= DXOR2;
               end
      DXOR2:   begin
                 if(raid_dv)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_addrin  <= xor_addrin + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= DXOR2;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   xor_ren    <= 1'b0;
                   state      <= DXOR3;
                 end
               end
       DXOR3:  begin
                 xor_wren <= 1'b0;
                 xor_addrin <= 11'h000;
                 if(block_cnt == 2'b01) 
/////////////////////////////////////////////////////////////////////
//this should become a parameter so we can do RAID5 or RAID6 DEMOS.
//01 is set up for RAID6 with 3 data disks, 10 would be for RAID5 with 
//4 data drives
/////////////////////////////////////////////////////////////////////
                 begin
                   block_cnt <= 2'b00;
                   state <= DLOAD;
                   memorydata_sel <= 2'b01;
                 end
                 else
                 begin
                   state <= DREQDATA1;
                   block_cnt <= block_cnt + 1'b1;
                 end
               end
      DLOAD:   begin
                 if (LOST ==11'b00000000100) //D0 lost
                 begin
                   raid_addr <= D0_ADDR;
                 end
                 else if (LOST ==11'b00000001000) //D1 lost
                 begin
                   raid_addr <= D1_ADDR;
                 end
                 else if (LOST ==11'b00000010000) //D2 lost
                 begin
                   raid_addr <= D2_ADDR;
                 end
                 else if (LOST ==11'b00000100000) //D3 lost
                 begin
                   raid_addr <= D3_ADDR;
                 end
                 else
                 begin
                   raid_addr <= D3_ADDR;
                 end
                 state       <= DLOAD1;
                 raid_req    <= 1'b1;
                 raid_RNW    <= 1'b0;
                 xor_addrout <= 11'h000;
                 xor_ren     <= 1'b1;
               end
      DLOAD1:  begin
                 if(raid_addrack)
                 begin
                   state <= DLOAD2;
                 end
                 else
                 begin
                   state <= DLOAD1;
                 end 
               end
      DLOAD2:  begin
                 state <= DLOAD3;
               end
      DLOAD3:  begin
                 state <= DLOAD4;
                 xor_addrout <= xor_addrout + 1'b1;
               end
      DLOAD4:  begin
                 state <= DLOAD5;
                 xor_addrout <= xor_addrout + 1'b1;
               end
      DLOAD5:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state <= DLOAD5;
                 end
                 else
                 begin
                   state       <= DONE;
                   cycle_done  <= 1'b1;
                   xor_ren     <= 1'b0;
                   xor_addrout <= 11'h000;
                 end
               end

//////////////////////////////////////////////////////////////////////////
//below are states for 2 disk lost cases
//////////////////////////////////////////////////////////////////////////
      DOUBLE:  begin
                 if(RAID_MODE == 3'b111) //PQ lost
                 begin
                   state     <= PQREQDATA;
                   raid_addr <= D0_ADDR;
                   raid_req  <= 1'b1;
                   raid_RNW       <= 1'b1;
                   bram0_muxsel_a <= 1'b1;
                   bram0_muxsel_b <= 1'b0;
                   bram0_wren     <= 1'b1;
                   xor_muxsel     <= 1'b0;
                   xor_wren       <= 1'b1;
                 end
                 else if(RAID_MODE == 3'b110) //DQ lost
                 begin
                   state     <= DQREQDATA;
                   raid_addr <= P_ADDR;
                   raid_req  <= 1'b1;
                   raid_RNW  <= 1'b1;
                   xor_muxsel <= 1'b0;
                   xor_wren   <= 1'b1;
                 end
                 else if(RAID_MODE == 3'b101) //DP lost
                 begin
                   casex(LOST) //which data disk??
                     //NEED to calc D0 
                     11'b000000001XX:  begin
                                         state     <= PDREQDATA;
                                         raid_addr <= D1_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end
                     //NEED to calc D1  
                     11'b000000010XX:  begin
                                         state     <= PDREQDATA;
                                         raid_addr <= D2_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end 
                     //NEED to calc D2 
                     11'b000000100XX:  begin
                                         state     <= PDREQDATA;
                                         raid_addr <= D0_ADDR;
                                         raid_req  <= 1'b1;
                                         raid_RNW  <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren   <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end 
                     //NEED to calc D3 //not supported 
                     //11'b000001000XX:  begin
                     //                    state     <= PDREQDATA;
                     //                    raid_addr <= P_ADDR;
                     //                    raid_req  <= 1'b1;
                     //                    raid_RNW  <= 1'b1;
                     //                    xor_muxsel <= 1'b0;
                     //                    xor_wren   <= 1'b1;
                     //                    bram0_wren <= 1'b1;
                     //                    bram0_muxsel_a <= 1'b1;
                     //                  end
                     //not valid 
                            default:   begin
                                         state <= IDLE;
                                       end
                   endcase
                 end
                 else  //DD lost
                 begin
                   dd_lost_mux <= 1'b1;
                   casex(LOST) //which data disk
                     //NEED to calc D0 & D1 
                     11'b000000011XX:  begin
                                         state <= DDREQDATA;
                                         raid_addr <= D2_ADDR;
                                         raid_req <= 1'b1;
                                         raid_RNW <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end
                     //NEED to calc D1 & D2  
                     11'b000000110XX:  begin
                                         state <= DDREQDATA;
                                         raid_addr <= D0_ADDR;
                                         raid_req <= 1'b1;
                                         raid_RNW <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end 
                     //NEED to calc D2 & D0 
                     11'b000000101XX:  begin
                                         state <= DDREQDATA;
                                         raid_addr <= D1_ADDR;
                                         raid_req <= 1'b1;
                                         raid_RNW <= 1'b1;
                                         xor_muxsel <= 1'b0;
                                         xor_wren <= 1'b1;
                                         bram0_wren <= 1'b1;
                                         bram0_muxsel_a <= 1'b1;
                                       end 
                     //NEED to calc D3 //not supported 
                     //11'b000001000XX:  begin
                     //                    //state <= DQREQDATA;
                     //                    raid_addr <= P_ADDR;
                     //                    raid_req <= 1'b1;
                     //                    raid_RNW <= 1'b1;
                     //                    xor_muxsel <= 1'b0;
                     //                    xor_wren <= 1'b1;
                     //                  end
                     //not valid 
                            default:   begin
                                         state <= IDLE;
                                       end
                   endcase
                 end
               end
    PQREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req <= 1'b0;
                   raid_RNW <= 1'b0;
                   mem_datain <= raid_rddata;
                   state <= PQXORMBWRITE;
                 end
                 else
                 begin
                   state <= PQREQDATA;
                 end
               end
 PQXORMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PQXORMBWRITE;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   bram0_wren   <= 1'b0;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b0;
                   state        <= PQXORMBWRITE1;
                 end
               end
PQXORMBWRITE1: begin
                 bram0_addrin <= 11'h000;
                 xor_addrin   <= 11'h000;
                 select_1     <= 3'b000;
                 bram0_ren    <= 1'b1;
                 bram0_muxsel_a <= 1'b0;
                 state          <= PQXORMBWRITE2;
               end
PQXORMBWRITE2: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state <= PQXORMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrin <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren <= 1'b1;
                     state <= PQXORMBWRITE2;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_addr     <= D1_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     xor_muxsel    <= 1'b1;
                     state         <= PQREQDATA1;
                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren <= 1'b1;
                   state <= PQXORMBWRITE2;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state <= PQXORMBWRITE2;
                 end                
               end
   PQREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PQXORMBWRITE3;
                 end
                 else
                 begin
                   state <= PQREQDATA1;
                 end                 
               end
PQXORMBWRITE3: begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PQXORMBWRITE4;
               end

PQXORMBWRITE4: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PQXORMBWRITE4;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_ren      <= 1'b0;
                   state        <= PQXORMBWRITE5;
                   bram2_wren   <= 1'b0;
                 end
               end
PQXORMBWRITE5: begin
                 xor_wren     <= 1'b0;
                 bram2_addrin <= 11'h000;
                 select_2     <= 3'b001;
                 bram2_ren    <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state          <= PQXORMBWRITE6;
               end
PQXORMBWRITE6: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= PQXORMBWRITE6;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= PQXORMBWRITE6;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     xor_addrout   <= 11'h000;
                     xor_addrin    <= 11'h000;
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram1_wren    <= 1'b0;
                     raid_addr     <= D2_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     state         <= PQREQDATA2;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram1_wren     <= 1'b1;
                   bram0_addrout  <= bram0_addrout + 1'b1;
                   state          <= PQXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= PQXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren <= 1'b1;
                   state <= PQXORMBWRITE6;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state <= PQXORMBWRITE6;
                 end                
               end
   PQREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PQXORMBWRITE7;
                 end
                 else
                 begin
                   state <= PQREQDATA2;
                 end                 
               end
PQXORMBWRITE7: begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PQXORMBWRITE8;
               end

PQXORMBWRITE8: begin
                if(raid_dv)
                 begin
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   bram2_addrin <= bram2_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PQXORMBWRITE8;
                 end
                 else
                 begin
                   xor_addrin   <= xor_addrin + 1'b1;
                   bram2_addrin <= bram2_addrin + 1'b1;
                   state        <= PQXORMBWRITE9;
                   bram2_wren   <= 1'b0;
                   xor_ren      <= 1'b0;
                 end
               end
PQXORMBWRITE9: begin
                 xor_wren     <= 1'b0;
                 bram2_addrin <= 11'h000;
                 select_2     <= 3'b010; 
                 bram2_ren    <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state          <= PQXORMBWRITE10;
               end
PQXORMBWRITE10: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= PQXORMBWRITE10;
                   end
                  else
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= PQXORMBWRITE10;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     //use this if only three data drives
                     bram2_wren  <= 1'b0;
                     raid_addr   <= P_ADDR;
                     raid_req    <= 1'b1;
                     raid_RNW    <= 1'b0;
                     bram1_addrout <= 11'h000;
                     xor_addrout   <= 11'h000;
                     xor_addrin    <= 11'h000;
                     state         <= PQLOAD;
                     memorydata_sel <= 2'b01;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram0_muxsel_b <= 1'b0;  
                     bram2_wren     <= 1'b0;
                     bram1_wren     <= 1'b0;
                     bram1_ren      <= 1'b1;

                     //if 4 drives this would continue
                     //bram2_wren <= 1'b0;
                     //raid_addr <= D3_ADDR;
                     //raid_req <= 1'b1;
                     //raid_RNW <= 1'b1;
                     //state    <= QREQDATA3;
                     //bram2_muxsel_a <= 1'b1;
                     //bram2_muxsel_b <= 1'b0;
                     //bram0_muxsel_b <= 1'b1;  //this allows bram1 data to get to the apprpriate XOR
                     //bram2_wren <= 1'b1;
                     //bram1_wren <= 1'b0;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram1_wren     <= 1'b1;
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   state          <= PQXORMBWRITE10;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_addrout <= bram1_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= PQXORMBWRITE10;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_ren     <= 1'b1;
                   state         <= PQXORMBWRITE10;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= PQXORMBWRITE10;
                 end                
               end
     PQLOAD:   begin
                 state     <= PQLOAD1;
                 raid_addr <= P_ADDR;
                 raid_req  <= 1'b1;
                 raid_RNW  <= 1'b0;
                 xor_addrout <= 11'h000;
                 xor_ren     <= 1'b1;
               end
//MD 3-22-06 skip PQLOAD2 
     PQLOAD1:  begin
                 if(raid_addrack)
                 begin
                   //state <= PQLOAD2;
                   state <= PQLOAD3;
                 end
                 else
                 begin
                   state <= PQLOAD1;
                 end 
               end

     PQLOAD2:  begin
                 state <= PQLOAD3;
               end
     PQLOAD3:  begin
                 state <= PQLOAD4;
               end
     PQLOAD4:  begin
                 state       <= PQLOAD5;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     PQLOAD5:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PQLOAD5;
                 end
                 else
                 begin
                   memorydata_sel <= 2'b10;
                   state          <= PQLOAD6;
                   xor_ren       <= 1'b0;
                   xor_addrout   <= 11'h000;
                   bram2_addrout <= 11'h000;
                   bram1_addrin <= 11'h000;
                   raid_addr    <= Q_ADDR;
                   raid_req     <= 1'b1;
                   raid_RNW     <= 1'b0;
                 end
               end
//MD 3-22-06 skip PQLOAD7 
     PQLOAD6:  begin
                 if(raid_addrack)
                 begin
                   //state <= PQLOAD7;
                     state <= PQLOAD8;
                 end
                 else
                 begin
                   state <= PQLOAD6;
                 end 
               end
     PQLOAD7:  begin
                 state <= PQLOAD8;
               end
     PQLOAD8:  begin
                 state <= PQLOAD9;
                 //MD 1_31_06 bram1_addrout <= bram1_addrout + 1'b1;
               end
     PQLOAD9:  begin
                 state <= PQLOAD10;
                 bram1_addrout <= bram1_addrout + 1'b1;
               end
     PQLOAD10:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   state         <= PQLOAD10;
                 end
                 else
                 begin
                   state         <= DONE;
                   cycle_done    <= 1'b1;
                   bram1_ren     <= 1'b0;
                   bram1_addrout <= 11'h000;
                 end
               end
    DQREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= DQXORBWRITE;
                 end
                 else
                 begin
                   state <= DQREQDATA;
                 end                 
               end
   DQXORBWRITE: begin
                 if(raid_dv)
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   mem_datain <= raid_rddata;
                   state      <= DQXORBWRITE;
                 end
                 else
                 begin
                   xor_addrin    <= xor_addrin + 1'b1;
                   state         <= DQREQDATA1;
                   xor_addrout   <= 11'h000;
                   bram0_muxsel_a <= 1'b1;
                   bram0_muxsel_b <= 1'b0;
                   bram0_wren    <= 1'b1;
                   xor_muxsel    <= 1'b0;
                   xor_wren      <= 1'b0;
                   raid_req      <= 1'b1;
                   raid_RNW      <= 1'b1;
                   if(LOST[5:2] == 4'b0001) //D0 lost
                   begin
                     raid_addr <= D1_ADDR;
                     select_1  <= 3'b001;
                   end
                   else if (LOST[5:2] == 4'b0010) //D1 lost
                   begin
                     raid_addr <= D2_ADDR;
                     select_1  <= 3'b010;
                   end
                    else if (LOST[5:2] == 4'b0100) //D2 lost
                   begin
                     raid_addr <= D0_ADDR;
                     select_1  <= 3'b000;
                   end
                   else //D3 lost(not supported)
                   begin
                   end
                 end
               end
   DQREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_addrin <= 11'h000;
                   xor_wren   <= 1'b1;
                   state      <= DQXORMBWRITE;
                 end
                 else
                 begin
                   state        <= DQREQDATA1;
                   xor_muxsel   <= 1'b1;
                   xor_ren      <= 1'b1;
                   bram0_addrin <= 11'h000;
                 end                 
               end
 DQXORMBWRITE: begin
                 bram0_addrin <= bram0_addrin + 1'b1;
                 xor_addrout  <= xor_addrout + 1'b1;
                 mem_datain   <= raid_rddata;
                 xor_wren     <= 1'b1;
                 state        <= DQXORMBWRITE1;
               end
 DQXORMBWRITE1: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= DQXORMBWRITE1;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   bram0_wren <= 1'b0;
                   xor_addrin <= xor_addrin + 1'b1;
                   state <= DQXORMBWRITE2;
                 end
               end
DQXORMBWRITE2: begin
                 bram0_addrin <= 11'h000;
                 xor_addrin   <= 11'h000;
                 xor_addrout  <= 11'h000;
                 //select_1 <= 3'b000;
                 bram0_ren    <= 1'b1;
                 xor_wren     <= 1'b0;
                 bram0_muxsel_a <= 1'b0;
                 state          <= DQXORMBWRITE3;
               end
DQXORMBWRITE3: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren    <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state        <= DQXORMBWRITE3;
                   end
                  else
                   begin
                     bram0_addrin  <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren    <= 1'b1;
                     state         <= DQXORMBWRITE3;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     xor_muxsel    <= 1'b1;
                     state         <= DQREQDATA2;
                     if(LOST[5:2] == 4'b0001) //D0 lost
                     begin
                       raid_addr <= D2_ADDR;
                       select_2  <= 3'b010;
                     end
                     else if (LOST[5:2] == 4'b0010) //D1 lost
                     begin
                       raid_addr <= D0_ADDR;
                       select_2  <= 3'b000;
                     end
                     else if (LOST[5:2] == 4'b0100) //D2 lost
                     begin
                       raid_addr <= D1_ADDR;
                       select_2  <= 3'b001;
                     end
                     else //D3 lost(not supported)
                     begin
                     end

                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren    <= 1'b1;
                   state         <= DQXORMBWRITE3;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= DQXORMBWRITE3;
                 end                
               end
   DQREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state      <= DQXORMBWRITE4;
                 end
                 else
                 begin
                   state        <= DQREQDATA2;
                   bram0_addrin <= 11'h000;
                 end                 
               end
DQXORMBWRITE4: begin
                 bram2_addrin <= bram2_addrin + 1'b1;
                 xor_addrout  <= xor_addrout + 1'b1;
                 xor_wren     <= 1'b1;
                 mem_datain   <= raid_rddata;
                 state        <= DQXORMBWRITE5;
               end

DQXORMBWRITE5: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= DQXORMBWRITE5;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_ren      <= 1'b0;
                   state        <= DQXORMBWRITE6;
                   bram2_wren   <= 1'b0;
                 end
               end
DQXORMBWRITE6: begin
                 xor_wren      <= 1'b0;
                 bram2_addrin  <= 11'h000;
                 bram2_ren     <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state         <= DQXORMBWRITE7;
               end
DQXORMBWRITE7: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= DQXORMBWRITE7;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= DQXORMBWRITE7;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     xor_addrout   <= 11'h000;
                     xor_addrin    <= 11'h000;
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram2_addrin  <= 11'h000;
                     bram1_wren    <= 1'b0;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b0;
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b0;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     state         <= DQLOAD;
                     if(LOST[5:2] == 4'b0001) //D0 lost
                     begin
                       raid_addr <= D0_ADDR;
                       select_2  <= 3'b010;
                     end
                     else if (LOST[5:2] == 4'b0010) //D1 lost
                     begin
                       raid_addr <= D1_ADDR;
                       select_2  <= 3'b000;
                     end
                     else if (LOST[5:2] == 4'b0100) //D2 lost
                     begin
                       raid_addr <= D2_ADDR;
                       select_2  <= 3'b001;
                     end
                     else
                     begin

                     end
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_wren    <= 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= DQXORMBWRITE7;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram0_addrout  <= bram0_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= DQXORMBWRITE7;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren     <= 1'b1;
                   state         <= DQXORMBWRITE7;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= DQXORMBWRITE7;
                 end                
               end
//MD 3-22-06 skip DQLOAD1 
      DQLOAD:  begin
                 if(raid_addrack)
                 begin
                   //state <= DQLOAD1;
                   state          <= DQLOAD2;
                   memorydata_sel <= 2'b01;
                 end
                 else
                 begin
                   state <= DQLOAD;
                 end 
               end
     DQLOAD1:  begin
                 state <= DQLOAD2;
               end
     DQLOAD2:  begin
                 state <= DQLOAD3;
               end
     DQLOAD3:  begin
                 state <= DQLOAD3A;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     DQLOAD3A:  begin
                 state       <= DQLOAD4;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     DQLOAD4:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout  <= xor_addrout + 1'b1;
                   bram2_addrin <= bram2_addrin + 1'b1;
                   state        <= DQLOAD4;
                   raid_req     <= 1'b0;
                 end
                 else
                 begin
                   memorydata_sel <= 2'b01;
                   state          <= DQMBWRITE;
                   xor_ren        <= 1'b0;
                   xor_addrout    <= 11'h000;
                   bram2_addrout  <= 11'h000;
                   bram2_wren     <= 1'b0;
                 end
               end
   DQMBWRITE: begin
                 bram2_addrin  <= 11'h000;
                 bram2_ren     <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state         <= DQMBWRITE1;
                 raid_addr     <= Q_ADDR;
                 if(LOST[5:2] == 4'b0001) //D0 lost
                     begin
                       select_2 <= 3'b000;
                     end
                     else if (LOST[5:2] == 4'b0010) //D1 lost
                     begin
                       select_2 <= 3'b001;
                       end
                     else if (LOST[5:2] == 4'b0100) //D2 lost
                     begin
                       select_2 <= 3'b010;
                     end
                     else
                     begin

                     end
               end
  DQMBWRITE1: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= DQMBWRITE1;
                   end
                  else
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= DQMBWRITE1;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     //use this if only three data drives
                     bram2_wren   <= 1'b0;
                     raid_addr    <= Q_ADDR;
                     raid_req     <= 1'b1;
                     raid_RNW     <= 1'b0;
                     bram1_addrout <= 11'h000;
                     state         <= DQLOAD5;
                     memorydata_sel <= 2'b10;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram0_muxsel_b <= 1'b0;  
                     bram2_wren    <= 1'b0;
                     bram1_wren    <= 1'b0;
                     bram1_ren     <= 1'b1;

                     //if 4 drives this would continue
                     //bram2_wren <= 1'b0;
                     //raid_addr  <= D3_ADDR;
                     //raid_req   <= 1'b1;
                     //raid_RNW   <= 1'b1;
                     //state      <= QREQDATA3;
                     //bram2_muxsel_a <= 1'b1;
                     //bram2_muxsel_b <= 1'b0;
                     //bram0_muxsel_b <= 1'b1;  //this allows bram1 data to get to the apprpriate XOR
                     //bram2_wren <= 1'b1;
                     //bram1_wren <= 1'b0;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram1_wren     <= 1'b1;
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   state          <= DQMBWRITE1;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state <= DQMBWRITE1;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_ren     <= 1'b1;
                   state         <= DQMBWRITE1;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= DQMBWRITE1;
                 end                
               end
     DQLOAD5:  begin
                 if(raid_addrack)
                 begin
                   state          <= DQLOAD6;
                   memorydata_sel <= 2'b10;
                 end
                 else
                 begin
                   state <= DQLOAD5;
                 end 
               end
     DQLOAD6:  begin
                 state <= DQLOAD7;
               end
     DQLOAD7:  begin
                 state         <= DQLOAD8;
                 bram1_addrout <= bram1_addrout + 1'b1;
               end
     DQLOAD8:  begin
                 state         <= DQLOAD9;
                 bram1_addrout <= bram1_addrout + 1'b1;
               end
     DQLOAD9:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   state         <= DQLOAD9;
                 end
                 else
                 begin
                   state         <= DONE;
                   cycle_done    <= 1'b1;
                   bram1_ren     <= 1'b0;
                   bram1_addrout <= 11'h000;
                   bram2_addrout <= 11'h000;
                   bram0_addrout <= 11'h000;
                   bram1_addrin  <= 11'h000;
                   bram0_addrin  <= 11'h000;
                 end
               end
    PDREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= PDXORMBWRITE;
                 end
                 else
                 begin
                   state <= PDREQDATA;
                 end
               end
 PDXORMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PDXORMBWRITE;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   bram0_wren   <= 1'b0;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b0;
                   state        <= PDXORMBWRITE1;
                 end
               end
PDXORMBWRITE1: begin
                 bram0_addrin <= 11'h000;
                 xor_addrin   <= 11'h000;
                 if(LOST[5:2] == 4'b0001) //D0 lost
                 begin
                   raid_addr <= D2_ADDR;
                   select_1  <= 3'b001;
                   select_2  <= 3'b010;
                 end
                 else if (LOST[5:2] == 4'b0010) //D1 lost
                 begin
                   raid_addr <= D0_ADDR;
                   select_1  <= 3'b010;
                   select_2  <= 3'b000;
                 end
                 else if (LOST[5:2] == 4'b0100) //D2 lost
                 begin
                   raid_addr <= D1_ADDR;
                   select_1  <= 3'b000;
                   select_2  <= 3'b001;
                 end
                 else //D3 lost(not supported)
                 begin
                 end
                 //select_1     <= 3'b000;
                 bram0_ren <= 1'b1;
                 bram0_muxsel_a <= 1'b0;
                 state          <= PDXORMBWRITE2;
               end
PDXORMBWRITE2: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren    <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state        <= PDXORMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrin  <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren    <= 1'b1;
                     state         <= PDXORMBWRITE2;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     xor_muxsel    <= 1'b1;
                     state         <= PDREQDATA1;
                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren    <= 1'b1;
                   state         <= PDXORMBWRITE2;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= PDXORMBWRITE2;
                 end                
               end
   PDREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   mem_datain  <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PDXORMBWRITE3;
                 end
                 else
                 begin
                   state <= PDREQDATA1;
                 end                 
               end
PDXORMBWRITE3: begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PDXORMBWRITE4;
               end

PDXORMBWRITE4: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= PDXORMBWRITE4;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_ren      <= 1'b0;
                   state        <= PDXORMBWRITE5;
                   bram2_wren   <= 1'b0;
                 end
               end
PDXORMBWRITE5: begin
                 xor_wren      <= 1'b0;
                 bram2_addrin  <= 11'h000;
                 xor_bypass    <= 1'b1;
                 //select_2 <= 3'b001;
                 bram2_ren     <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state         <= PDXORMBWRITE6;
               end
PDXORMBWRITE6: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= PDXORMBWRITE6;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= PDXORMBWRITE6;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     xor_addrout   <= 11'h000;
                     xor_addrin    <= 11'h000;
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram1_wren    <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram1_ren     <= 1'b1;
                     raid_addr     <= Q_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     state         <= PDREQDATA2;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_wren    <= 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= PDXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram0_addrout  <= bram0_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= PDXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren     <= 1'b1;
                   state         <= PDXORMBWRITE6;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= PDXORMBWRITE6;
                 end                
               end
   PDREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req       <= 1'b0;
                   raid_RNW       <= 1'b0;
                   mem_datain     <= raid_rddata;
                   mem_datain_reg <= mem_datain;
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   state          <= PDXORMBWRITE7;
                   if(LOST[5:2] == 4'b0001) //D0 lost
                   begin
                     select_3 <= 4'b0000;
                   end
                   else if (LOST[5:2] == 4'b0010) //D1 lost
                   begin
                     select_3 <= 4'b0001;
                   end
                   else if (LOST[5:2] == 4'b0100) //D2 lost
                   begin
                     select_3 <= 4'b0010;
                   end
                   else
                   begin
                   end
                 end
                 else
                 begin
                   state <= PDREQDATA2;
                 end                 
               end
PDXORMBWRITE7: begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   //xor_addrout <= xor_addrout + 1'b1;
                   bram3_addrin  <= 11'h000;
                   bram3_wren    <= 1'b1;
                   bram2_wren    <= 1'b0;
                   mem_datain    <= raid_rddata;
                   mem_datain_reg <= mem_datain;
                   state         <= PDXORMBWRITE8;
               end

PDXORMBWRITE8: begin
                 mem_datain_reg <= mem_datain;
                 if(raid_dv)
                 begin
                 mem_datain <= raid_rddata;
                   if (bram1_addrout > 11'h006)
                   begin                   
                     bram3_wren    <= 1'b1;
                     bram3_addrin  <= bram3_addrin + 1'b1;
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= PDXORMBWRITE8;
                   end
                   else if(bram1_addrout == 11'h006) 
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram3_wren    <= 1'b1;
                     state         <= PDXORMBWRITE8;
                   end
                   else if(bram1_addrout == 11'h005)
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= PDXORMBWRITE8;
                   end
                   else
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= PDXORMBWRITE8;
                   end                
                 end
                 else
                 begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   bram3_addrin  <= bram3_addrin + 1'b1;
                   state         <= PDXORMBWRITE9;
                   bram1_ren     <= 1'b0;
                 end
               end
PDXORMBWRITE9: begin
                 mem_datain_reg <= mem_datain;
                 if(bram3_addrin == DATABLOCKSIZE)
                 begin
                   bram3_wren    <= 1'b0;
                   bram3_addrin  <= 11'h000;
                   state         <= PDLOAD;
                   raid_req      <= 1'b1;
                   raid_RNW      <= 1'b0;
                   memorydata_sel <= 2'b11;
                   if(LOST[5:2] == 4'b0001) //D0 lost
                   begin
                     raid_addr <= D0_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0010) //D1 lost
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0100) //D2 lost
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   else
                   begin
                   end
                 end
                 else
                 begin
                   bram3_addrin <= bram3_addrin + 1'b1;
                   state <= PDXORMBWRITE9;
                 end
               end
     PDLOAD:   begin
                 state         <= PDLOAD1;
                 raid_req      <= 1'b1;
                 raid_RNW      <= 1'b0;
                 bram3_addrout <= 11'h000;
                 bram3_ren     <= 1'b1;
                 xor_addrout   <= 11'h000;
                 feedback_loop <= 1'b1;
                 xor_ren       <= 1'b1;
               end
     PDLOAD1:  begin
                 if(raid_addrack)
                 begin
                   state <= PDLOAD2;
                 end
                 else
                 begin
                   state <= PDLOAD1;
                 end 
               end
     PDLOAD2:  begin
                 state <= PDLOAD3;
               end

     PDLOAD3:  begin
                 state         <= PDLOAD4;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_muxsel    <= 1'b1;
               end
     PDLOAD4:  begin
                 state         <= PDLOAD5;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_addrout   <= xor_addrout +1'b1;
               end
     PDLOAD5:  begin
                 state         <= PDLOAD6;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_addrout   <= xor_addrout +1'b1;
                 xor_wren      <= 1'b1;
               end

     PDLOAD6:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram3_addrout <= bram3_addrout + 1'b1;
                   xor_addrout   <= xor_addrout +1'b1;
                   xor_addrin    <= xor_addrin +1'b1;
                   state         <= PDLOAD6;
                 end
                 else
                 begin
                   state          <= PDLOAD7;
                   xor_wren       <= 1'b0;
                   xor_ren        <= 1'b0;
                   xor_addrout    <= 11'h000;
                   xor_addrin     <= xor_addrin +1'b1;
                   memorydata_sel <= 2'b01;
                   feedback_loop  <= 1'b0;
                   bram3_ren      <= 1'b0;
                   raid_addr      <= P_ADDR;
                   raid_req       <= 1'b1;
                   raid_RNW       <= 1'b0;
                 end
               end
//MD 3-23-06 skip PDLOAD2 
     PDLOAD7:  begin
                 if(raid_addrack)
                 begin
                   //state <= PDLOAD8;
                   state <= PDLOAD9;
                   xor_ren <= 1'b1;
                 end
                 else
                 begin
                   state <= PDLOAD7;
                 end 
               end
     PDLOAD8:  begin
                 state <= PDLOAD9;
               end
     PDLOAD9:  begin
                 state <= PDLOAD10;
                 
               end
     PDLOAD10:  begin
                 state <= PDLOAD11;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     PDLOAD11:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= PDLOAD11;
                 end
                 else
                 begin
                   state          <= DONE;
                   cycle_done     <= 1'b1;
                   bram1_ren      <= 1'b0;
                   bram1_addrout  <= 11'h000;
                   xor_addrout    <= 11'h000;
                   xor_ren        <= 1'b0;
                   xor_bypass     <= 1'b0;
                   bram0_muxsel_a <= 1'b0;
                   bram0_muxsel_b <= 1'b0;
                   xor_addrin     <= 11'h000;
                   bram0_addrin   <= 11'h000;
                 end
               end
    DDREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= DDXORMBWRITE;
                 end
                 else
                 begin
                   state <= DDREQDATA;
                 end
               end
 DDXORMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= DDXORMBWRITE;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   bram0_wren   <= 1'b0;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b0;
                   state        <= DDXORMBWRITE1;
                 end
               end
DDXORMBWRITE1: begin
                 bram0_addrin <= 11'h000;
                 xor_addrin   <= 11'h000;
                 if(LOST[5:2] == 4'b0011) //D0 D1 lost
                   begin
                     select_1 <= 3'b010;
                   end
                   else if (LOST[5:2] == 4'b0110) //D1 D2 lost
                   begin
                     select_1 <= 3'b000;
                   end
                   else if (LOST[5:2] == 4'b0101) //D2 D0 lost
                   begin
                     select_1 <= 3'b001;
                   end
                 bram0_ren      <= 1'b1;
                 bram0_muxsel_a <= 1'b0;
                 state          <= DDXORMBWRITE2;
               end
DDXORMBWRITE2: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren    <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state        <= DDXORMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrin  <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren    <= 1'b1;
                     state         <= DDXORMBWRITE2;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_addr     <= P_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     xor_muxsel    <= 1'b1;
                     state         <= DDREQDATA1;
                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren    <= 1'b1;
                   state         <= DDXORMBWRITE2;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= DDXORMBWRITE2;
                 end                
               end
   DDREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   mem_datain  <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= DDXOR;
                 end
                 else
                 begin
                   state <= DDREQDATA1;
                 end                 
               end
     DDXOR:    begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   xor_addrout <= xor_addrout + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= DDXOR1;
                   xor_wren    <= 1'b1;
                 end
                 else
                 begin
                   state <= DDXOR;
                 end                 
               end
     DDXOR1:   begin
                 xor_addrout <= xor_addrout + 1'b1;
                 mem_datain  <= raid_rddata;
                 xor_addrin  <= xor_addrin + 1'b1;
                 state       <= DDXOR2;
               end
     DDXOR2:   begin
                 if(raid_dv)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_addrin  <= xor_addrin + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= DDXOR2;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   xor_ren    <= 1'b0;
                   state      <= DDXOR3;
                 end
               end
      DDXOR3:  begin
                 xor_wren       <= 1'b0;
                 xor_addrin     <= 11'h000;
                 state          <= DDREQDATA2;
                 raid_addr      <= Q_ADDR;
                 raid_req       <= 1'b1;
                 raid_RNW       <= 1'b1;
                 bram2_muxsel_a <= 1'b1;
                 bram2_muxsel_b <= 1'b0;
                 bram2_wren     <= 1'b1;
               end
   DDREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= DDMBWRITE;
                 end
                 else
                 begin
                   state <= DDREQDATA2;
                 end                 
               end
    DDMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= DDMBWRITE;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   state        <= DDMBWRITE1;
                   bram2_wren   <= 1'b0;
                 end
               end
   DDMBWRITE1: begin
                 bram2_addrin <= 11'h000;
                 select_2     <= 3'b000;
                 bram2_ren    <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state        <= DDMBWRITE2;
               end
   DDMBWRITE2: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= DDMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= DDMBWRITE2;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram1_wren    <= 1'b0;
                     if(LOST[5:2] == 4'b0011) //D0 D1 lost
                     begin
                       select_0 <= 3'b001;
                       select_3 <= 4'b1000;
                     end
                     else if (LOST[5:2] == 4'b0110) //D1 D2 lost
                     begin
                       select_0 <= 3'b010;
                       select_3 <= 4'b1001;
                     end
                     else if (LOST[5:2] == 4'b0101) //D2 D0 lost
                     begin
                       select_0 <= 3'b000;
                       select_3 <= 4'b1010;
                     end
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     xor_bypass     <= 1'b0;
                     xor_ren        <= 1'b1;
                     xor_addrout    <= 11'h000;
                     state          <= DDCALCDATA;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram1_wren    <= 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= DDMBWRITE2;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout  <= bram2_addrout + 1'b1;
                   bram0_addrout  <= bram0_addrout + 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state          <= DDMBWRITE2;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren     <= 1'b1;
                   state         <= DDMBWRITE2;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state <= DDMBWRITE2;
                 end                
               end
   DDCALCDATA: begin
                 xor_addrout <= xor_addrout + 1'b1;
                 state <= DDCALCDATA1;
               end
  DDCALCDATA1: begin
                 xor_addrout <= xor_addrout + 1'b1;
                 state <= DDCALCDATA2;
               end
  DDCALCDATA2: begin
                 if(xor_addrout == 11'h03)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= DDCALCDATA3;
                 end
                 else
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= DDCALCDATA2;
                 end
               end
  DDCALCDATA3: begin
                 bram1_ren   <= 1'b1;
                 xor_addrout <= xor_addrout + 1'b1;
                 state       <= DDCALCDATA4;
               end
  DDCALCDATA4: begin
                 bram1_addrout <= bram1_addrout  + 1'b1;
                 xor_addrout   <= xor_addrout + 1'b1;
                 bram3_muxsel_a <= 1'b1; //MD 3_22_06
                 bram3_wren    <= 1'b1;
                 bram3_addrin  <= 11'h000;
                 state         <= DDCALCDATA5;
               end 
  DDCALCDATA5: begin
                 if(bram1_addrout == DATABLOCKSIZE)
                 begin
                   xor_ren       <= 1'b0;
                   bram1_ren     <= 1'b0;
                   bram1_addrout <= 11'h000;
                   xor_addrout   <= 11'h000;
                   bram3_addrin  <= bram3_addrin + 1'b1;
                   state         <= DDCALCDATA6;
                 end
                 else
                 begin 
                   if(bram1_addrout > 11'h006)
                   begin
                     bram3_addrin <= bram3_addrin + 1'b1;
                   end
                   bram1_addrout <= bram1_addrout  + 1'b1;
                   xor_addrout   <= xor_addrout + 1'b1;
                   state         <= DDCALCDATA5;
                 end
               end
  DDCALCDATA6: begin
                 if(bram3_addrin == DATABLOCKSIZE)
                 begin
                   bram3_wren <= 1'b0;
                   state <= DDLOAD;
                   if(LOST[5:2] == 4'b0011) //D0 D1 lost
                   begin
                     raid_addr <= D0_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0110) //D1 D2 lost
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0101) //D2 D0 lost
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   raid_req       <= 1'b1;
                   raid_RNW       <= 1'b0;
                   xor_addrin     <= 11'h000;
                   memorydata_sel <= 2'b11;
                   bram0_muxsel_b <= 1'b0;  
                 end
                 else
                 begin 
                   bram3_addrin <= bram3_addrin + 1'b1;
                   state        <= DDCALCDATA6;
                 end
               end     
     DDLOAD:   begin
                 state         <= DDLOAD1;
                 raid_req      <= 1'b1;
                 raid_RNW      <= 1'b0;
                 bram3_addrout <= 11'h000;
                 bram3_ren     <= 1'b1;
                 xor_addrout   <= 11'h000;
                 feedback_loop <= 1'b1;
                 xor_ren       <= 1'b1;
               end
     DDLOAD1:  begin
                 if(raid_addrack)
                 begin
                   state <= DDLOAD2;
                 end
                 else
                 begin
                   state <= DDLOAD1;
                 end 
               end
     DDLOAD2:  begin
                 state <= DDLOAD3;
               end
     DDLOAD3:  begin
                 state         <= DDLOAD4;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_muxsel    <= 1'b1;
               end
     DDLOAD4:  begin
                 state         <= DDLOAD5;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_addrout   <= xor_addrout +1'b1;
               end
     DDLOAD5:  begin
                 state         <= DDLOAD6;
                 bram3_addrout <= bram3_addrout + 1'b1;
                 xor_addrout   <= xor_addrout +1'b1;
                 xor_wren      <= 1'b1;
               end

     DDLOAD6:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram3_addrout <= bram3_addrout + 1'b1;
                   xor_addrout   <= xor_addrout +1'b1;
                   xor_addrin    <= xor_addrin +1'b1;
                   state         <= DDLOAD6;
                 end
                 else
                 begin
                   state          <= DDLOAD7;
                   xor_wren       <= 1'b0;
                   xor_ren        <= 1'b0;
                   xor_addrout    <= 11'h000;
                   xor_addrin     <= xor_addrin +1'b1;
                   memorydata_sel <= 2'b01;
                   feedback_loop  <= 1'b0;
                   bram3_ren      <= 1'b0;
                   if(LOST[5:2] == 4'b0011) //D0 D1 lost
                   begin
                     raid_addr <= D1_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0110) //D1 D2 lost
                   begin
                     raid_addr <= D2_ADDR;
                   end
                   else if (LOST[5:2] == 4'b0101) //D2 D0 lost
                   begin
                     raid_addr <= D0_ADDR;
                   end
                   raid_req <= 1'b1;
                   raid_RNW <= 1'b0;
                 end
               end
     DDLOAD7:  begin
                 if(raid_addrack)
                 begin
                   state   <= DDLOAD8;
                   xor_ren <= 1'b1;
                 end
                 else
                 begin
                   state <= DDLOAD7;
                 end 
               end
     DDLOAD8:  begin
                 state <= DDLOAD9;
               end
     DDLOAD9:  begin
                 state        <= DDLOAD10;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     DDLOAD10:  begin
                 state       <= DDLOAD11;
                 xor_addrout <= xor_addrout + 1'b1;
               end
     DDLOAD11:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= DDLOAD11;
                 end
                 else
                 begin
                   state         <= DONE;
                   cycle_done    <= 1'b1;
                   bram1_ren     <= 1'b0;
                   bram1_addrout <= 11'h000;
                   xor_addrout   <= 11'h000;
                   xor_ren       <= 1'b0;
                   xor_bypass    <= 1'b0;
                   bram0_muxsel_a <= 1'b0;
                   bram0_muxsel_b <= 1'b0;
                   xor_addrin    <= 11'h000;
                   bram0_addrin  <= 11'h000;
                   bram3_addrin  <= 11'h000;
                   dd_lost_mux   <= 1'b0;
                 end
               end
////////////////////////////////////////////////////////////////////////////
//this is updating P & Q when writing data
///////////////////////////////////////////////////////////////////////////
     UPDATE:   begin
                 state          <= UREQDATA;
                 raid_addr      <= D0_ADDR;
                 raid_req       <= 1'b1;
                 raid_RNW       <= 1'b1;
                 xor_muxsel     <= 1'b0;
                 xor_wren       <= 1'b1;
                 bram0_muxsel_a <= 1'b1;
                 bram0_muxsel_b <= 1'b0;
                 bram0_wren     <= 1'b1;
                 //xor_muxsel   <= 1'b0;
                 //xor_wren     <= 1'b1;
                 case(LOST)
                     //Update because of D0 
                     11'b00000000100:  begin
                                         select_1 <= 3'b000;
                                         select_2 <= 3'b000;
                                       end
                     //Update because of D1  
                     11'b00000001000:  begin
                                         select_1 <= 3'b001;
                                         select_2 <= 3'b001;                
                                       end 
                     //Update because of D2 
                     11'b00000010000:  begin
                                         select_1 <= 3'b010;
                                         select_2 <= 3'b010;
                                       end 
                 endcase
               end 
     UREQDATA: begin
                 if(raid_dv)
                 begin
                   raid_req   <= 1'b0;
                   raid_RNW   <= 1'b0;
                   mem_datain <= raid_rddata;
                   state      <= UXORMBWRITE;
                 end
                 else
                 begin
                   state <= UREQDATA;
                 end
               end
  UXORMBWRITE: begin
                 if(raid_dv)
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= UXORMBWRITE;
                 end
                 else
                 begin
                   bram0_addrin <= bram0_addrin + 1'b1;
                   bram0_wren   <= 1'b0;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b0;
                   state        <= UXORMBWRITE1;
                 end
               end
UXORMBWRITE1: begin
                 bram0_addrin   <= 11'h000;
                 xor_addrin     <= 11'h000;
                 bram0_ren      <= 1'b1;
                 bram0_muxsel_a <= 1'b0;
                 state          <= UXORMBWRITE2;
               end
UXORMBWRITE2: begin
                 if (bram0_addrout > 11'h006)
                 begin                   
                   if (bram0_addrout == DATABLOCKSIZE)
                   begin
                     bram0_ren    <= 1'b0;
                     bram0_addrin <= bram0_addrin + 1'b1;
                     state        <= UXORMBWRITE2;
                   end
                  else
                   begin
                     bram0_addrin  <= bram0_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram0_wren    <= 1'b1;
                     state         <= UXORMBWRITE2;
                   end
                   if (bram0_addrin == DATABLOCKSIZE)
                   begin
                     bram0_wren    <= 1'b0;
                     bram0_addrout <= 11'h000;
                     raid_addr     <= D1_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b1;
                     xor_ren       <= 1'b1;
                     xor_muxsel    <= 1'b1;
                     state         <= UREQDATA1;
                   end
                 end
                 else if(bram0_addrout == 11'h006)
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   bram0_wren    <= 1'b1;
                   state         <= UXORMBWRITE2;
                 end 
                 else
                 begin
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= UXORMBWRITE2;
                 end                
               end
    UREQDATA1: begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   mem_datain  <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= UXORMBWRITE3;
                 end
                 else
                 begin
                   state <= UREQDATA1;
                 end                 
               end
 UXORMBWRITE3: begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= UXORMBWRITE4;
               end
 UXORMBWRITE4: begin
                if(raid_dv)
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrout  <= xor_addrout + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_wren     <= 1'b1;
                   mem_datain   <= raid_rddata;
                   state        <= UXORMBWRITE4;
                 end
                 else
                 begin
                   bram2_addrin <= bram2_addrin + 1'b1;
                   xor_addrin   <= xor_addrin + 1'b1;
                   xor_ren      <= 1'b0;
                   state        <= UXORMBWRITE5;
                   bram2_wren   <= 1'b0;
                 end
               end
 UXORMBWRITE5: begin
                 xor_wren       <= 1'b0;
                 bram2_addrin   <= 11'h000;
                 bram2_ren      <= 1'b1;
                 bram2_muxsel_a <= 1'b0;
                 state          <= UXORMBWRITE6;
               end
 UXORMBWRITE6: begin
                 if (bram2_addrout > 11'h006)
                 begin                   
                   if (bram2_addrout == DATABLOCKSIZE)
                   begin
                     bram2_ren     <= 1'b0;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram0_addrout <= bram0_addrout + 1'b1;
                     state         <= UXORMBWRITE6;
                   end
                  else
                   begin
                     bram0_addrout <= bram0_addrout + 1'b1;
                     bram1_addrin  <= bram1_addrin + 1'b1;
                     bram2_addrout <= bram2_addrout + 1'b1;
                     bram1_wren    <= 1'b1;
                     state         <= UXORMBWRITE6;
                   end
                   if (bram1_addrin == DATABLOCKSIZE)
                   begin
                     xor_addrout   <= 11'h000;
                     xor_addrin    <= 11'h000;
                     bram0_addrout <= 11'h000;
                     bram0_ren     <= 1'b0;
                     bram1_addrin  <= 11'h000;
                     bram2_addrout <= 11'h000;
                     bram1_wren    <= 1'b0;
                     raid_addr     <= P_ADDR;
                     raid_req      <= 1'b1;
                     raid_RNW      <= 1'b1;
                     bram0_muxsel_b <= 1'b1;  
                     bram2_muxsel_a <= 1'b1;
                     bram2_muxsel_b <= 1'b0;
                     bram2_wren    <= 1'b0; 
                     xor_ren       <= 1'b1;
                     state         <= UREQDATA2;
                   end
                 end
                 else if(bram2_addrout == 11'h006)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   //bram1_addrin <= bram1_addrin + 1'b1;
                   bram1_wren    <= 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   state         <= UXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h005)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_addrout <= bram0_addrout + 1'b1;
                   
                   bram1_wren     <= 1'b1;
                   bram1_muxsel_a <= 1'b0;
                   state         <= UXORMBWRITE6;
                 end
                 else if(bram2_addrout == 11'h004)
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   bram0_ren     <= 1'b1;
                   state         <= UXORMBWRITE6;
                 end
                 else
                 begin
                   bram2_addrout <= bram2_addrout + 1'b1;
                   state         <= UXORMBWRITE6;
                 end                
               end
////////////////////////////////////////////////////////////////////
//we are now getting the old P
////////////////////////////////////////////////////////////////////
    UREQDATA2: begin
                 if(raid_dv)
                 begin
                   raid_req    <= 1'b0;
                   raid_RNW    <= 1'b0;
                   mem_datain  <= raid_rddata;
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= UXORMBWRITE7;
                 end
                 else
                 begin
                   state <= UREQDATA2;
                 end                 
               end
 UXORMBWRITE7: begin
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_wren    <= 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= UXORMBWRITE8;
               end

UXORMBWRITE8: begin
                if(raid_dv)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   xor_addrin  <= xor_addrin + 1'b1;
                   mem_datain  <= raid_rddata;
                   state       <= UXORMBWRITE8;
                 end
                 else
                 begin
                   xor_addrin <= xor_addrin + 1'b1;
                   state      <= UXORMBWRITE9;
                   xor_ren    <= 1'b0;
                 end
               end
UXORMBWRITE9: begin
                 xor_wren       <= 1'b0;
                 bram2_addrin   <= 11'h000;
                 bram2_ren      <= 1'b0;
                 bram2_muxsel_a <= 1'b0;
                 state          <= ULOAD;
               end
      ULOAD:   begin
                 state          <= ULOAD1;
                 raid_addr      <= P_ADDR;
                 raid_req       <= 1'b1;
                 raid_RNW       <= 1'b0;
                 xor_addrout    <= 11'h000;
                 xor_ren        <= 1'b1;
                 memorydata_sel <= 2'b01; //MD 2_2_06
               end
      ULOAD1:  begin
                 if(raid_addrack)
                 begin
                   state <= ULOAD2;
                 end
                 else
                 begin
                   state <= ULOAD1;
                 end 
               end
      ULOAD2:  begin
                 state <= ULOAD3;
               end
     ULOAD3:  begin
                 state       <= ULOAD4;
                 //MD 1_31_06 xor_addrout <= xor_addrout + 1'b1;
                 //added it back in on 2_8-06
                 xor_addrout <= xor_addrout + 1'b1;
               end
      ULOAD4:  begin
                 state       <= ULOAD5;
                 xor_addrout <= xor_addrout + 1'b1;
               end
      ULOAD5:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   xor_addrout <= xor_addrout + 1'b1;
                   state       <= ULOAD5;
                 end
                 else
                 begin
                   xor_addrout    <= 11'h000;
                   xor_addrin     <= 11'h000;
                   bram0_addrout  <= 11'h000;
                   bram0_ren      <= 1'b0;
                   bram1_addrin   <= 11'h000;
                   bram2_addrout  <= 11'h000;
                   bram1_wren     <= 1'b0;
                   bram1_ren      <= 1'b1;
                   raid_addr      <= Q_ADDR;
                   raid_req       <= 1'b1;
                   raid_RNW       <= 1'b1;
                   bram0_muxsel_b <= 1'b1;  
                   bram2_muxsel_a <= 1'b1;
                   bram2_muxsel_b <= 1'b0;
                   xor_ren        <= 1'b0; 
                   state          <= UREQDATA3;
                 end
               end  
   UREQDATA3: begin
                 if(raid_dv)
                 begin
                   raid_req       <= 1'b0;
                   raid_RNW       <= 1'b0;
                   mem_datain     <= raid_rddata;
                   mem_datain_reg <= mem_datain;
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   state          <= UXORMBWRITE10;
                 end
                 else
                 begin
                   state <= UREQDATA3;
                 end                 
               end
UXORMBWRITE10: begin
                   bram1_addrout  <= bram1_addrout + 1'b1;
                   //xor_addrout <= xor_addrout + 1'b1;
                   bram3_addrin   <= 11'h000;
                   bram3_muxsel_a <= 1'b0;
                   bram3_wren     <= 1'b1;
                   bram2_wren     <= 1'b0;
                   mem_datain     <= raid_rddata;
                   mem_datain_reg <= mem_datain;
                   state          <= UXORMBWRITE11;
               end

UXORMBWRITE11: begin
                 mem_datain_reg <= mem_datain;
                 if(raid_dv)
                 begin
                 mem_datain <= raid_rddata;
                   if (bram1_addrout > 11'h006)
                   begin                   
                     bram3_wren    <= 1'b1;
                     bram3_addrin  <= bram3_addrin + 1'b1;
                     bram1_addrout <= bram1_addrout + 1'b1;
                     state         <= UXORMBWRITE11;
                   end
                   else if(bram1_addrout == 11'h006) 
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram3_addrin  <= bram3_addrin + 1'b1;
                     bram3_wren    <= 1'b1;
                     state         <= UXORMBWRITE11;
                   end
                   else if(bram1_addrout == 11'h005)
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram3_addrin  <= bram3_addrin + 1'b1;
                     state         <= UXORMBWRITE11;
                   end
                   else
                   begin
                     bram1_addrout <= bram1_addrout + 1'b1;
                     bram3_addrin  <= bram3_addrin + 1'b1;
                     state         <= UXORMBWRITE11;
                   end                
                 end
                 else
                 begin
                   bram1_addrout <= bram1_addrout + 1'b1;
                   bram3_addrin  <= bram3_addrin + 1'b1;
                   state         <= UXORMBWRITE12;
                   bram1_ren     <= 1'b0;
                 end
               end
UXORMBWRITE12: begin
                 mem_datain_reg <= mem_datain;
                 if(bram3_addrin == DATABLOCKSIZE)
                 begin
                   bram3_wren     <= 1'b0;
                   bram3_addrin   <= 11'h000;
                   bram3_muxsel_a <= 1'b0;
                   raid_req       <= 1'b1;
                   raid_RNW       <= 1'b0;
                   raid_addr      <= Q_ADDR;
                   bram3_ren      <= 1'b1;
                   bram3_addrout  <= 11'h000;
                   state          <= ULOAD6;
                   memorydata_sel <= 2'b11;
                 end
                 else
                 begin
                   bram3_addrin <= bram3_addrin + 1'b1;
                   state        <= UXORMBWRITE12;
                 end
               end
     ULOAD6:  begin
                 if(raid_addrack)
                 begin
                   state <= ULOAD7;
                 end
                 else
                 begin
                   state <= ULOAD6;
                 end 
               end
     ULOAD7:  begin
                 state <= ULOAD8;
               end
     ULOAD8:  begin
                 state         <= ULOAD9;
                 //added it in on 2_8_2006
                 bram3_addrout <= bram3_addrout + 1'b1;
               end
     ULOAD9:  begin
                 state         <= ULOAD10;
                 bram3_addrout <= bram3_addrout + 1'b1;
               end
     ULOAD10:  begin
                 raid_req <= 1'b0;
                 if(raid_wrrdy)
                 begin
                   bram3_addrout <= bram3_addrout + 1'b1;
                   state         <= ULOAD10;
                 end
                 else
                 begin
                   state         <= DONE;
                   cycle_done    <= 1'b1;
                   bram3_ren     <= 1'b0;
                   bram3_addrout <= 11'h000;
                 end
               end
      DONE:    begin
                 xor_muxsel     <= 1'b0;
                 xor_addrin     <= 11'h000;
                 xor_addrout    <= 11'h000;
                 xor_wren       <= 1'b0;
                 xor_ren        <= 1'b0;
                 bram0_muxsel_a <= 1'b0;        
                 bram0_muxsel_b <= 1'b0; 
                 bram1_muxsel_a <= 1'b0;        
                 bram1_muxsel_b <= 1'b0;
                 bram2_muxsel_a <= 1'b0;        
                 bram2_muxsel_b <= 1'b0;
                 bram3_muxsel_a <= 1'b1;
                 bram0_addrin   <= 11'h000;
                 bram0_addrout  <= 11'h000;
                 bram1_addrin   <= 11'h000;
                 bram1_addrout  <= 11'h000;
                 bram2_addrin   <= 11'h000;
                 bram2_addrout  <= 11'h000;
                 bram3_addrin   <= 11'h000;
                 bram3_addrout  <= 11'h000;
                 bram0_wren     <= 1'b0;
                 bram0_ren      <= 1'b0;
                 bram1_wren     <= 1'b0;
                 bram1_ren      <= 1'b0;
                 bram2_wren     <= 1'b0;
                 bram2_ren      <= 1'b0;
                 bram3_wren     <= 1'b0;
                 bram3_ren      <= 1'b0;
                 select_0       <= 3'b000;
                 select_1       <= 3'b000;
                 select_2       <= 3'b000;
                 select_3       <= 4'b0000;
                 raid_req       <= 1'b0;
                 raid_addr      <= 32'h00000000;
                 raid_RNW       <= 1'b1;
                 block_cnt      <= 2'b00;
                 memorydata_sel <= 2'b00;
                 mem_datain_reg <= 32'h00000000;
                 feedback_loop  <= 1'b0;
                 dd_lost_mux    <= 1'b0;
                 cycle_done     <= 1'b0;
                 state          <= IDLE;
               end
      default: begin
                 state <= IDLE;
               end

    endcase
  end
end

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//this state machine loade the DCR registers into the appropriate registers
reg   [2:0] DCR_state;
reg  [10:0] sector_count; 
//sector_count is used support sector sizes larger than the 512byte 
parameter   DCR_START   = 3'b000;
parameter   DCR_GET     = 3'b001;
parameter   DCR_DONE    = 3'b010;
parameter   DCR_DONE1   = 3'b011;
parameter   DCR_DONE2   = 3'b100;

parameter   DCR_REPEAT  = 3'b101;
always @ (posedge clk or posedge rst)
begin
  if(rst)
  begin
    DCR_state <= DCR_START;
    DCR_sel   <= 2'b00;
    LOST      <= 11'h000;
    D0_ADDR   <= 32'h00000000;
    D1_ADDR   <= 32'h00000000;
    D2_ADDR   <= 32'h00000000;
    D3_ADDR   <= 32'h00000000;
    P_ADDR    <= 32'h00000000;
    Q_ADDR    <= 32'h00000000;
    RAID_MODE <= 3'b000; //defined in port list
    sector_count <= 11'h000;
    DCR_done     <= 1'b0;
    done         <= 1'b0;
  end
  else
  begin
    case (DCR_state)
      DCR_START: begin
                   if(start)
                   begin
                     DCR_state <= DCR_GET;
                     LOST      <= DCR_reg0;
                     RAID_MODE <= DCR_reg1;
                     DCR_sel   <= 2'b01;
                     case(RAID_SIZE[7:0])
                         8'b00000001: begin //512 sector
                                        //sector_count <= 11'h003;
                                        sector_count <= 11'h000;
                                      end
                         8'b00000010: begin //1k sector
                                        sector_count <= 11'h007;
                                      end
                         8'b00000100: begin //2K sector
                                        sector_count <= 11'h00F;
                                      end
                         8'b00001000: begin //4k sector
                                        sector_count <= 11'h01F;
                                      end
                         8'b00010000: begin //8k sector
                                        sector_count <= 11'h03F;
                                      end
                         8'b00100000: begin //16k sector
                                        sector_count <= 11'h07F;
                                      end
                         8'b01000000: begin //32k sector
                                        sector_count <= 11'h0FF;
                                      end
                         8'b10000000: begin //64k sector
                                        sector_count <= 11'h1FF;
                                      end
                         default:     begin
                                        sector_count <= 11'h003;
                                      end
                     endcase
                   end
                   else
                   begin
                     DCR_state <= DCR_START;
                   end
                 end
      DCR_GET:   begin
                   if(DCR_sel == 2'b01)
                   begin
                     DCR_state <= DCR_GET;
                     D0_ADDR   <= DCR_reg0;
                     D3_ADDR   <= DCR_reg1;
                     DCR_sel   <= 2'b10;
                   end
                   else if(DCR_sel == 2'b10)
                   begin
                     DCR_state <= DCR_GET;
                     D1_ADDR   <= DCR_reg0;
                     P_ADDR    <= DCR_reg1;
                     DCR_sel   <= 2'b11;
                   end 
                   else if(DCR_sel == 2'b11)
                   begin
                     DCR_state <= DCR_DONE;
                     D2_ADDR   <= DCR_reg0;
                     Q_ADDR    <= DCR_reg1;
                     DCR_sel   <= 2'b00;
                     DCR_done  <= 1'b1;
                   end 
                 end
      DCR_DONE:  begin
                   if(cycle_done)
                   begin
                     if(sector_count == 11'b000)
                     begin
                       DCR_state <= DCR_DONE1;
                       DCR_done  <= 1'b0;
                       done      <= 1'b1;
                     end
                     else
                     begin
                       DCR_state <= DCR_REPEAT;
                       sector_count <= sector_count - 1'b1;
                       D0_ADDR   <= D0_ADDR + 8'h80;
                       D1_ADDR   <= D1_ADDR + 8'h80;
                       D2_ADDR   <= D2_ADDR + 8'h80;
                       D3_ADDR   <= D3_ADDR + 8'h80;
                       P_ADDR    <= P_ADDR + 8'h80;
                       Q_ADDR    <= Q_ADDR + 8'h80;
 
                      DCR_done  <= 1'b0;
                     end
                   end
                   else
                   begin
                     DCR_state <= DCR_DONE;
                   end
                 end
     DCR_DONE1:  begin
                   DCR_state <= DCR_DONE2;
                   DCR_done  <= 1'b0;
                   //done <= 1'b0;
                 end
     DCR_DONE2:  begin
                   DCR_state <= DCR_START;
                   DCR_done  <= 1'b0;
                   done      <= 1'b0;
                 end
     DCR_REPEAT: begin
                   DCR_state <= DCR_DONE;
                   DCR_done  <= 1'b1;
                 end
        default: begin
                   DCR_state <= DCR_START;
                   DCR_sel   <= 2'b00;
                   LOST      <= 11'h000;
                   D0_ADDR   <= 32'h00000000;
                   D1_ADDR   <= 32'h00000000;
                   D2_ADDR   <= 32'h00000000;
                   D3_ADDR   <= 32'h00000000;
                   P_ADDR    <= 32'h00000000;
                   Q_ADDR    <= 32'h00000000;
                   RAID_MODE <= 3'b000; 
                   DCR_done  <= 1'b0;
                 end

    endcase  
  end
end

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//datapath logic below
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//this indicates that the BRAM are loaded with the GFLOG and GFILOG tables
assign gfc_done = ~rst;

//this are xor that are needed external to the buliding blocks below
assign xor_data_1 = mux_data_0 ^ mult_data_2;
assign xor_data_2 = mux_data_1 ^ mem_datain_reg;
assign xor_data_3 = mux_data_4 ^ mult_data_0;
assign xor_data_4 = xor_data_2 ^ mult_data_3;

assign mux_data_3 = xor_bypass ? xor_data_2 : xor_data_3;
assign mux_data_4 = dd_lost_mux ? mux_data_1 : xor_data_2;

//determines if XOR data for such things as parity or other cases
//may create bigger mux later on //MD
//00 goes through all of the logic
//01 comes from XORBRAM
//10 comes from the MB1
//11 comes from the MB3
assign raid_wrdata = memorydata_sel[1] ? (memorydata_sel[0] ? bram_data_0: mux_data_1): 
                                         (memorydata_sel[0] ? xor_data_0: xor_data_4);  

//this is the info to be put into the memory
//the assignment will change depending on what is being created
assign mem_dataout = xor_data_4;

assign xor_bramdatain = feedback_loop ? bram_data_0 : mem_datain;


////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
//modular block that work on 8bits of data
////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////


genvar i;
   generate for(i = 0; i < `raid_width; i = i+8)
   begin: raid6_8bit_datapath_inst 

XOR_BRAM xbi_0(
   .datain        (xor_bramdatain[i+7:i]),
   .clk           (clk), 
   .rst           (rst),
   .muxsel        (xor_muxsel), 
   .bram_in_addr  (xor_addrin), 
   .bram_in_wren  (xor_wren), 
   .bram_out_addr (xor_addrout), 
   .bram_out_ren  (xor_ren), 
   .dataout       (xor_data_0[i+7:i]));

MUX_BRAM_BLOCK mbi_0(
   .bramdatain_a  (mem_datain[i+7:i]), 
   .bramdatain_b  (mult_data_1[i+7:i]),
   .muxdatain     (mux_data_1[i+7:i]),  
   .clk           (clk), 
   .muxsel_a      (bram0_muxsel_a), 
   .muxsel_b      (bram0_muxsel_b),
   .bram_in_addr  (bram0_addrin), 
   .bram_in_wren  (bram0_wren), 
   .bram_out_addr (bram0_addrout), 
   .bram_out_ren  (bram0_ren), 
   .dataout       (mux_data_0[i+7:i]));

MUX_BRAM_BLOCK mbi_1(
   .bramdatain_a  (mem_datain[i+7:i]), 
   .bramdatain_b  (xor_data_1[i+7:i]),
   .muxdatain     (8'h00),  
   .clk           (clk), 
   .muxsel_a      (bram1_muxsel_a), 
   .muxsel_b      (bram1_muxsel_b),
   .bram_in_addr  (bram1_addrin), 
   .bram_in_wren  (bram1_wren), 
   .bram_out_addr (bram1_addrout), 
   .bram_out_ren  (bram1_ren), 
   .dataout       (mux_data_1[i+7:i]));

MUX_BRAM_BLOCK mbi_2(
   .bramdatain_a  (mem_datain[i+7:i]), 
   .bramdatain_b  (xor_data_0[i+7:i]),//use for DQ case
   .muxdatain     (8'h00),  
   .clk           (clk), 
   .muxsel_a      (bram2_muxsel_a), 
   .muxsel_b      (bram2_muxsel_b),
   .bram_in_addr  (bram2_addrin), 
   .bram_in_wren  (bram2_wren), 
   .bram_out_addr (bram2_addrout), 
   .bram_out_ren  (bram2_ren), 
   .dataout       (mux_data_2[i+7:i]));

RAID_MULT_4 rmi_0(
   .dspin_a       (dspin_0), 
   .dspin_b       (dspin_1),
   .dspin_c       (dspin_2), 
   .dspin_d       (dspin_3),
   .gflogin_a     (xor_data_0[i+7:i]), 
   .gflogin_b     (mux_data_0[i+7:i]),
   .gflogin_c     (mux_data_2[i+7:i]), 
   .gflogin_d     (mux_data_3[i+7:i]),
   .gflog_ren_a   (1'b0),//gflog_ren_a),
   .gflog_ren_b   (1'b0),//gflog_ren_b), 
   .gflog_ren_c   (1'b0),//gflog_ren_c),
   .gflog_ren_d   (1'b0),//gflog_ren_d),
   .gfilog_ren_a  (1'b0),//gfilog_ren_a),
   .gfilog_ren_b  (1'b0),//gfilog_ren_b),
   .gfilog_ren_c  (1'b0),//gfilog_ren_c),
   .gfilog_ren_d  (1'b0),//gfilog_ren_d),  
   .clk           (clk), 
   .rst           (rst),
   .dd_lost       (dd_lost_mux),
   .gfilogout_a   (mult_data_0[i+7:i]), 
   .gfilogout_b   (mult_data_1[i+7:i]),
   .gfilogout_c   (mult_data_2[i+7:i]), 
   .gfilogout_d   (mult_data_3[i+7:i]));

MUX_BRAM_BLOCK mbi_3(
   .bramdatain_a  (mult_data_3[i+7:i]),
   .bramdatain_b  (xor_data_2[i+7:i]),
   .muxdatain     (8'h00),  
   .clk           (clk), 
   .muxsel_a      (bram3_muxsel_a),
   .muxsel_b      (1'b0),
   .bram_in_addr  (bram3_addrin), 
   .bram_in_wren  (bram3_wren), 
   .bram_out_addr (bram3_addrout), 
   .bram_out_ren  (bram3_ren), 
   .dataout       (bram_data_0[i+7:i]));

   end  
 endgenerate


mig_mem_if b_if(
.clk            (clk),
.rst            (rst),
.m_wrdata       (mem_wrdata),     //`raid_width 
.m_wr_parity    (mem_wr_parity),
.m_rddata       (mem_rddata),     //`raid_width extra bits are set to zero because only a 64 bit raid can fit in the ML555 platform
.m_rd_parity    (mem_rd_parity),
.m_addr         (mem_addr),       //36
.m_data_wen     (mem_data_wen),
.m_addr_wen     (mem_addr_wen),
.m_data_valid   (mem_data_valid),
.raid_req       (raid_req),
.raid_addr      (raid_addr),      //32
.raid_RNW       (raid_RNW),
.raid_wrdata    ({raid_wrdata, raid_wrdata}),    //`raid_width
.sector_len     ({1'b0,DATABLOCKSIZE}),
.raid_addrack   (raid_addrack),
.raid_wrrdy     (raid_wrrdy),
.raid_dv        (raid_dv),
.raid_rddata    (raid_rddata),    //`raid_width
.raid_complete  (raid_complete),
.single_error   (SINGLE_ERR),
.double_error   (DOUBLE_ERR)
); 


//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//this has the g constants for calculating the Q in the GF multiplication
//////////////////////////////////////////////////////////////////////////
hard_code_mux hcm0(
   .select     (select_0), 
   .hard_data  (dspin_0));
hard_code_mux hcm1(
   .select     (select_1), 
   .hard_data  (dspin_1));
hard_code_mux hcm2(
   .select     (select_2), 
   .hard_data  (dspin_2));
hard2_code_mux hcm3(
   .select     (select_3), 
   .hard_data  (dspin_3));
	
endmodule
